/*
 * Copyright (c) 1996-1998 Regents of the University of California.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the SPAND Research
 *      Group at the University of California Berkeley.
 * 4. Neither the name of the University nor of the Research Group may be
 *    used to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
 
/* $Header: /home/stemm/cvs/SPAND/javasrc/ContentNeg/ChunkTokenizer.java,v 1.2 1998/05/28 20:09:04 stemm Exp $ */
/* Divides a string into "chunks", where a chunk is delimited by the
characters specified in the constructor. */
/* Example: ChunkTokenizer("{type text/html} {features tables}", "{}")
   will return the strings "type text/html" and "features tables" */


public class ChunkTokenizer {

  char[] s;
  char startChar, endChar;
  int curIndex;
  int length;
  
  ChunkTokenizer(String s0, String delim) {
    s = new char[s0.length()];
    s0.getChars(0, s0.length(), s, 0);
    length=s0.length();
    startChar=delim.charAt(0);
    endChar=delim.charAt(1);
    curIndex=0;
  }

  public String nextToken() {
    /* Beginning and end indices of chunk (inclusive) */
    int startIndex, endIndex;

    /* Depth counts chunks inside chunks */
    int depth;

    if (curIndex>=length){
      return null;
    }
    
    /* Look for the character startChar */
    for(startIndex=curIndex; s[startIndex]!=startChar; startIndex++){
      if (startIndex>=length){
	return null;
      }
    }
    startIndex++;

    depth=1;
    /* Look for the character endChar that makes depth 0 again*/
    for(endIndex=startIndex; depth>0; endIndex++){
      if (endIndex>=length){
	return null;
      }
      if (s[endIndex]==endChar){
	depth--;
      }
      if (s[endIndex]==startChar){
	depth++;
      }
    }
    curIndex=endIndex;
    endIndex--;

    /* Extract the substring into a new string */
    return new String(s, startIndex, endIndex-startIndex);
  }
}

  

  
