/*
 * Copyright (c) 1996-1998 Regents of the University of California.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the SPAND Research
 *      Group at the University of California Berkeley.
 * 4. Neither the name of the University nor of the Research Group may be
 *    used to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
/* $Header: /home/stemm/cvs/SPAND/javasrc/ContentNeg/ConNegFilter.java,v 1.3 1998/05/28 20:09:04 stemm Exp $ */
/* ConNegFilter.java */

import muffin.*;
import java.util.*;
import java.lang.*;
import java.net.*;
import java.io.*;

public class ConNegFilter implements ContentFilter
{
    Prefs prefs;
    ConNeg factory;
  InputStream in;
  OutputStream out;
  TCNAlternate pickAlt;
  
    ConNegFilter (ConNeg factory)
    {
	this.factory = factory;
    }
    
    public void setPrefs (Prefs prefs)
    {
	this.prefs = prefs;
    }

    public void setInputStream (InputStream in)
    {
	this.in = in;
    }

    public void setOutputStream (OutputStream out)
    {
	this.out = out;
    }

    public boolean needsFiltration (Request req, Reply r) 
    {
      Vector altVector = new Vector();
      String alts, altString, tmp;
      Enumeration altE;
      TCNAlternate firstAlt = null, tmpAlt;
      
      /* Skip this entirely unless the content-type is image/jpeg or image/gif */
      if (r.getHeaderField("Content-type").compareTo("image/jpeg")==0 ||
	  r.getHeaderField("Content-type").compareTo("image/gif")==0){
      
	/* If there is a response header named TCN: then the resource is
	   transparently negotiated. This client only negotiates on
	   content length, so if the list of alternates includes
	   information about content length, then we select the version
	   whose content length is closest to the fractional size we
	   want. We assume that the largest alternate is the full-size image. */
	
	if (r.getHeaderField("TCN")!=null){
	  if ((alts=r.getHeaderField("Alternates"))==null){
	    /* Shouldn't get here, if TCN is present alternates must be too */
	    System.err.println("TCN resource without alternates?");
	  }
	  
	  /* Parse the list of alternates, inserting them into a linked list sorted by quality */
	  StringTokenizer tok = new StringTokenizer(alts, ",");
	  while (tok.hasMoreTokens()){
	    altString=tok.nextToken();
	    TCNAlternate alt=new TCNAlternate(altString);
	    if (firstAlt==null){
	      firstAlt=alt;
	    } else {
	      /* Walk the list of alternates, inserting this one at the right place */
	      for(tmpAlt=firstAlt; ; tmpAlt=tmpAlt.next){
		if (tmpAlt.next==null){
		  /* Insert at end and quit */
		  tmpAlt.next=alt;
		  alt.next=null;
		  break;
		} else if (tmpAlt.next.quality>alt.quality){
		  alt.next=tmpAlt.next;
		  tmpAlt.next=alt;
		  break;
		}
	      }
	    }
	  }
	  
	  Integer frac = new Integer(prefs.getString("ConNeg.reduceFrac"));
	  pickAlt=firstAlt;
	  for(tmpAlt=firstAlt; tmpAlt!=null; tmpAlt=tmpAlt.next){
	    /* DEBUG: Print out the lengths in the list of alternates */
	    System.err.println("uri="+tmpAlt.URI);
	    System.err.println("quality="+tmpAlt.quality);
	    //System.out.println("type="+tmpAlt.type);
	    //	  System.out.println("length="+tmpAlt.length);
	    
	    /* Find the alternate whose size is closest to the one we want */
	    /* We assume that the largest image is the original one */
	    //	    if (Math.abs((double) (tmpAlt.size-firstAlt.size*frac.intValue()))<
	    //		Math.abs((double) (pickAlt.size-firstAlt.size*frac.intValue()))){
	    //	      pickAlt=tmpAlt;
	    
	    /* Find the alternate whose quality is closest to the one we want */
	    if (Math.abs((double) (tmpAlt.quality-(frac.intValue()/100.0)))<
		Math.abs((double) (pickAlt.quality-(frac.intValue()/100.0)))){
	      pickAlt=tmpAlt;
	    }
	  }

	  /* Actually retrieve the alternate that is closest */
	  System.err.println("Picking "+pickAlt.URI);
	  
	  return true;
	}
      }
      return false;
    }

  public void run ()
  {
    int b;
    URL url;
    
    /* Actually retrieve the alternate that is closest */
	  System.err.println("Picking "+pickAlt.URI);
    try {
      System.err.println("host="+prefs.getString ("muffin.httpProxyHost"));
      System.err.println("port="+prefs.getString ("muffin.httpProxyPort"));
      if((prefs.getString ("muffin.httpProxyHost")).length () > 0
	 && prefs.getInteger ("muffin.httpProxyPort") > 0){
	url = new URL("http", prefs.getString("muffin.httpProxyHost"),
		      prefs.getInteger ("muffin.httpProxyPort"), pickAlt.URI);
      } else {
	url = new URL(pickAlt.URI);
      }
      
      System.err.println("2");
      InputStream in = url.openStream();

      /* Write the input to the output */
      while((b=in.read())!=-1){
	out.write(b);
      }
      out.flush();
      out.close();
      
      System.err.println("4");
    }
    catch (MalformedURLException e)
      {
	System.err.println("Malformed URL "+pickAlt.URI);
      }
    catch (IOException e)
      {
	System.err.println("Couldn't download URL "+pickAlt.URI);
      }
  }

}
