/*
 * Copyright (c) 1996-1998 Regents of the University of California.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the SPAND Research
 *      Group at the University of California Berkeley.
 * 4. Neither the name of the University nor of the Research Group may be
 *    used to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
/* $Header: /home/stemm/cvs/SPAND/javasrc/ContentNeg/ConNegFrame.java,v 1.3 1998/05/28 20:09:04 stemm Exp $ */
/* ConNegFrame.java */

import java.awt.*;
import java.awt.event.*;
import muffin.*;

public class ConNegFrame extends Frame implements ActionListener, WindowListener
{
    Prefs prefs;
    ConNeg parent;
    CheckboxGroup reduceFrac;
    Checkbox twenty, fourty, sixty, eighty, hundred;
    Label title;
  
    public ConNegFrame (Prefs prefs, ConNeg parent)
    {
	super ("ConNeg");

	this.prefs = prefs;
	this.parent = parent;

	setBackground (Color.lightGray);
	setFont (new Font ("Helvetica", Font.BOLD, 12));

	Panel panel = new Panel ();
	GridBagLayout layout = new GridBagLayout ();
	panel.setLayout (layout);
	GridBagConstraints c;

	title = new Label("Percentage size of images (relative to old size):");
	c = new GridBagConstraints ();
	c.gridwidth = GridBagConstraints.REMAINDER;
	layout.setConstraints (title, c);
	panel.add(title);
	
	reduceFrac = new CheckboxGroup();

	hundred = new Checkbox("100", reduceFrac, false);
	panel.add(hundred);

	eighty = new Checkbox("80", reduceFrac, false);
	panel.add(eighty);

	sixty = new Checkbox("60", reduceFrac, false);
	panel.add(sixty);

	fourty = new Checkbox("40", reduceFrac, false);
	panel.add(fourty);

	twenty = new Checkbox("20", reduceFrac, false);
	panel.add(twenty);

	/* Get the percentage in the prefs. If none exists, use 100% */
	if (prefs.getString("ConNeg.reduceFrac")==null){
	  reduceFrac.setSelectedCheckbox(hundred);
	} else {
	  Integer bar = new Integer(prefs.getString("ConNeg.reduceFrac"));
	  switch (bar.intValue()){
	  case 100:
	    reduceFrac.setSelectedCheckbox(hundred);
	    break;
	  case 80: 
	    reduceFrac.setSelectedCheckbox(eighty);
	    break;
	  case 60: 
	    reduceFrac.setSelectedCheckbox(sixty);
	    break;
	  case 40: 
	    reduceFrac.setSelectedCheckbox(fourty);
	    break;
	  case 20: 
	    reduceFrac.setSelectedCheckbox(twenty);
	    break;
	  default: 
	    reduceFrac.setSelectedCheckbox(hundred);
	    break;
	  }
	}
	
	add ("Center", panel);

	Button b;
	Panel buttonPanel = new Panel ();
	buttonPanel.setLayout (new GridLayout (1, 4));
	b = new Button ("Apply");
	b.setActionCommand ("doApply");
	b.addActionListener (this);
	buttonPanel.add (b);
	b = new Button ("Save");
	b.setActionCommand ("doSave");
	b.addActionListener (this);
	buttonPanel.add (b);
	b = new Button ("Clear");
	b.setActionCommand ("doClear");
	b.addActionListener (this);
	buttonPanel.add (b);
	b = new Button ("Close");
	b.setActionCommand ("doClose");
	b.addActionListener (this);
	buttonPanel.add (b);

	add ("South", buttonPanel);

	addWindowListener (this);
	
	pack ();
	setSize (getPreferredSize ());

	show ();
    }

    public void actionPerformed (ActionEvent event)
    {
	String arg = event.getActionCommand ();
	
	if ("doApply".equals (arg)){
	  Integer foo=new Integer(reduceFrac.getSelectedCheckbox().getLabel());
	  prefs.putInteger("ConNeg.reduceFrac", foo.intValue());
	} else if ("doSave".equals (arg)){
	  parent.save ();
	} else if ("doClose".equals (arg)){
	    setVisible (false);
	}
    }

    public void windowActivated (WindowEvent e)
    {
    }
  
    public void windowDeactivated (WindowEvent e)
    {
    }
  
    public void windowClosing (WindowEvent e)
    {
	setVisible (false);
    }
  
    public void windowClosed (WindowEvent e)
    {
    }
  
    public void windowIconified (WindowEvent e)
    {
    }
  
    public void windowDeiconified (WindowEvent e)
    {
    }
  
    public void windowOpened (WindowEvent e)
    {
    }
}
