/*
 * Copyright (c) 1996-1998 Regents of the University of California.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the SPAND Research
 *      Group at the University of California Berkeley.
 * 4. Neither the name of the University nor of the Research Group may be
 *    used to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
/* $Header: /home/stemm/cvs/SPAND/javasrc/ContentNeg/TCNAlternate.java,v 1.2 1998/05/28 20:09:04 stemm Exp $ */
import java.util.*;

/* Structure that contains a Transparent Content Negotiation Alternate */
public class TCNAlternate {

  TCNAlternate next; /* Reference to next alternate */
  String URI;
  float quality;
  String type;
  String language;
  int length;
  String charset;
  String description;
  Hashtable features; /* Hash from feature tag to value (both strings) */

  /* Given a string tokenizer and a delimiting character, constructs
     the remaining (unparsed) part of the string */
  String RemainingString(StringTokenizer tok, String delim)
  {
    String res = new String("");
    
    while (tok.hasMoreTokens()){
      res=res.concat(tok.nextToken());
      if (tok.hasMoreTokens()){
	res=res.concat(delim);
      }
    }
    return res;
  }
  
  /* Constructor from string (including enclosing { and }). Parses value */
  TCNAlternate(String s)
  {
    String tmpStr;

    length=0;
    
    ChunkTokenizer tmpTok = new ChunkTokenizer(s, "{}");
    String actual=tmpTok.nextToken();
    
    StringTokenizer tok = new StringTokenizer(actual, " ");

    URI=tok.nextToken();
    /* The first string is the URI, possibly quoted. Remove the quotes */
    StringTokenizer tmpTok2 = new StringTokenizer(URI, "\"");
    URI=tmpTok2.nextToken();

    /* The second string is the quality as a float. */
    Float tmp=new Float(tok.nextToken());
    quality=tmp.floatValue();
    
    /* The Following strings are key-value pairs, enclosed by {}s. */
    ChunkTokenizer chunk = new ChunkTokenizer(actual, "{}");
    while ((tmpStr = chunk.nextToken()) != null){
      StringTokenizer tok1 = new StringTokenizer(tmpStr);
      String key = tok1.nextToken();

      /* Figure out what the key is (XXX only handle type, length for now) */
      if (key.compareTo(new String("type"))==0){
	//	length=Integer.parseInt(tmpTok.nextToken());
	type=tok1.nextToken();
      }
      else if (key.compareTo(new String("length"))==0){
      	length=Integer.parseInt(tok1.nextToken());
      }
    }
  }
}
   
