package graph;

import java.awt.*;
import java.applet.*;
import java.net.URL;
import java.util.*;
import java.io.*;

/*
**************************************************************************
**
**    Class  AxisPrinter
**
**************************************************************************
** Copyright (c) 1998
**      The Regents of the University of California.  All rights reserved.
**
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions
** are met:
** 1. Redistributions of source code must retain the above copyright
**    notice, this list of conditions and the following disclaimer.
** 2. Redistributions in binary form must reproduce the above copyright
**    notice, this list of conditions and the following disclaimer in the
**    documentation and/or other materials provided with the distribution.
** 3. All advertising materials mentioning features or use of this software
**    must display the following acknowledgement:
**      This product includes software developed by the University of
**      California, Berkeley and its contributors.
** 4. Neither the name of the University nor the names of its contributors
**    may be used to endorse or promote products derived from this software
**    without specific prior written permission.
**
** THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
** ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
** IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
** ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
** FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
** DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
** OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
** HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
** LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
** OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
** SUCH DAMAGE.
**
** Written by Mark Stemm (stemm@cs.berkeley.edu)
**************************************************************************
**
**    This class takes as input an Axis and can generate postscript
**    commands that show the Axis.
**
*************************************************************************/


/**
 * This class takes as input an Axis and generates postscript using a
 * "print to file" method. It depends on postscript commands that are
 * defined earlier in the postscript file (by the GraphPrinter object's
 * drawToFile method).
 * @version $Revision: 1.2 $, $Date: 1998/04/13 22:20:42 $.
 * @author Mark Stemm (stemm@cs.berkeley.edu)
 */
 
public class AxisPrinter {

      /*
*******************
** Public Variables
******************/
  /**
   * The Axis object to generate the postscript for.
   */
  Axis a; 
  
    /*
*******************
** Constructors
******************/

  public AxisPrinter(Axis a0){
    a=a0;
  }

  /*
*******************
** Public Methods
******************/
  
  /**
   * Output postcript commands to the stream file that draw the
   * axis. The variables otherMin and otherMax indicate the line along
   * which the axis should be drawn (for example, if this axis is a Y
   * (vertical) axis, then draw the axis along the lines (x=otherMin) and (x=otherMax).
   */
  public void drawToFile(PrintStream file, double otherMin, double otherMax)
  {
    int i;
    
    // Draw the Basic Axis
      switch (a.position){
    case a.LEFT: case a.RIGHT:
      file.println(otherMin+" xunits "+a.minimum+" yunits "+" moveto");
      file.println(otherMin+" xunits "+a.maximum+" yunits "+" lineto");
      file.println("stroke");
      file.println(otherMax+" xunits "+a.minimum+" yunits "+" moveto");
      file.println(otherMax+" xunits "+a.maximum+" yunits "+" lineto");
      file.println("stroke");
      break;
    case a.BOTTOM: case a.TOP:
      file.println(a.minimum+" xunits "+otherMin+" yunits "+" moveto");
      file.println(a.maximum+" xunits "+otherMin+" yunits "+" lineto");
      file.println("stroke");
      file.println(a.minimum+" xunits "+otherMax+" yunits "+" moveto");
      file.println(a.maximum+" xunits "+otherMax+" yunits "+" lineto");
      file.println("stroke");
      break;
    }

    // Now Draw tick marks for the axes
    // Only Draw the tick mark if between the min and max
    for(i=0; i<a.label_count-1; i++){
      if (a.label_start+(i*a.label_step)<=a.maximum &&
	  a.label_start+(i*a.label_step)>=a.minimum){
	switch (a.position){
	case a.LEFT:
	  file.println(otherMin+" xunits "+(a.label_start+(i*a.label_step))+" yunits moveto");
	  file.println("5 0 rlineto");
	  file.println("stroke");
	  file.println(otherMin+" xunits "+(a.label_start+(i*a.label_step))+" yunits moveto");
	  file.println("-25 0 rmoveto");
	  file.println("("+(a.label_start+(i*a.label_step))+") show");
	  break;
	case a.RIGHT:
	  file.println(otherMax+" xunits "+(a.label_start+(i*a.label_step))+" yunits moveto");
	  file.println("-5 0 rlineto");
	  file.println("stroke");
	  file.println(otherMax+" xunits "+(a.label_start+(i*a.label_step))+" yunits moveto");
	  file.println("25 0 rmoveto");
	  file.println("("+(a.label_start+(i*a.label_step))+") show");
	  break;
	case a.BOTTOM:
	  file.println((a.label_start+(i*a.label_step))+" xunits "+otherMin+" yunits moveto");
	  file.println("0 5 rlineto");
	  file.println("stroke");
	  file.println((a.label_start+(i*a.label_step))+" xunits "+otherMin+" yunits moveto");
	  file.println("0 -15 rmoveto");
	  file.println("("+(a.label_start+(i*a.label_step))+") show");
	  break;
	case a.TOP:
	  file.println((a.label_start+(i*a.label_step))+" xunits "+otherMax+" yunits moveto");
	  file.println("0 5 rlineto");
	  file.println("stroke");
	  file.println((a.label_start+(i*a.label_step))+" xunits "+otherMax+" yunits moveto");
	  file.println("0 -15 rmoveto");
	  file.println("("+(a.label_start+(i*a.label_step))+") show");
	  break;
	}
      }
    }

    // Now Draw the label for the axis
    switch (a.position) {
    case a.LEFT:
      file.println("gsave");
      file.println("newpath");
      file.println(otherMin+" xunits "+(a.minimum+(a.maximum-a.minimum)/3)+" yunits "+" moveto");
      file.println("-40 0 rmoveto");
      file.println("90 rotate");
      file.println("("+a.title.getText()+") show");
      file.println("grestore");
      break;
    case a.RIGHT:
      file.println("gsave");
      file.println("newpath");
      file.println(otherMax+" xunits "+(a.minimum+(a.maximum-a.minimum)/3)+" yunits "+" moveto");
      file.println("40 0 rmoveto");
      file.println("90 rotate");
      file.println("("+a.title.getText()+") show");
      file.println("grestore");
      break;
    case a.BOTTOM:
      file.println("gsave");
      file.println("newpath");
      file.println((a.minimum+(a.maximum-a.minimum)/3)+" xunits "+otherMin+" yunits "+" moveto");
      file.println("0 -40 rmoveto");
      file.println("("+a.title.getText()+") show");
      file.println("grestore");
      break;
    case a.TOP:
      file.println("gsave");
      file.println("newpath");
      file.println((a.minimum+(a.maximum-a.minimum)/3)+" xunits "+otherMax+" yunits "+" moveto");
      file.println("0 40 rmoveto");
      file.println("("+a.title.getText()+") show");
      file.println("grestore");
      break;
    }
  }
}
       



