package graph;

import java.awt.*;
import java.util.*;
import java.lang.*;
import java.io.*;
import java.net.URL;

/*
**************************************************************************
**
**    Class MarkersPrinter
**
**************************************************************************
** Copyright (c) 1998
**      The Regents of the University of California.  All rights reserved.
**
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions
** are met:
** 1. Redistributions of source code must retain the above copyright
**    notice, this list of conditions and the following disclaimer.
** 2. Redistributions in binary form must reproduce the above copyright
**    notice, this list of conditions and the following disclaimer in the
**    documentation and/or other materials provided with the distribution.
** 3. All advertising materials mentioning features or use of this software
**    must display the following acknowledgement:
**      This product includes software developed by the University of
**      California, Berkeley and its contributors.
** 4. Neither the name of the University nor the names of its contributors
**    may be used to endorse or promote products derived from this software
**    without specific prior written permission.
**
** THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
** ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
** IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
** ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
** FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
** DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
** OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
** HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
** LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
** OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
** SUCH DAMAGE.
**
** Written by Mark Stemm (stemm@cs.berkeley.edu)
**************************************************************************
**
**    This class takes as input a Markers class and can generate
**    postscript commands that show the marker.
**
*************************************************************************/

/**
 *    This class takes as input a Markers class and generates
 *    postscript using a "print to file" method. It depends on
 *    postscript commands that are defined earlier in the postscript
 *    file (by the Gra[hPrinter object's drawToFile method).
 *    @version $Revision: 1.1 $, $Date: 1998/04/13 22:20:46 $.
 *    @author Mark Stemm (stemm@cs.berkeley.edu)
 */

public class MarkersPrinter {

      /*
*******************
** Public Variables
******************/
  /**
   * The Markers object to generate the postscript for.
   */
  Markers m; 
  
    /*
*******************
** Constructors
******************/

  public MarkersPrinter(Markers m0){
    m=m0;
  }

  /*
*******************
** Public Methods
******************/
  
  /**
   * draw the marker
   * @param file File to print to
   * @param m Index of the marker to draw
   * @param scale scale factor. All coordinates are multiplied by this factor.
   * @param x Coordinate where to draw the marker
   * @param y Coordinate where to draw the marker
   */
  public void drawToFile(PrintStream file, int mindex, double scale, int x, int y) {
    int i;
    MarkerVertex mv;
    int x0 = x, x1 = x, y0 = y, y1 = y;
    Vector v;
          
          
    if(mindex < 1 || mindex > m.max ) return;
    if(scale <= 0) return;
          
    v = m.vert[mindex-1];
    if( v == null) return;

    for (i=0; i<v.size(); i++) {
      mv = (MarkerVertex)v.elementAt(i);

      if( mv.draw ) {
	x1 = x + (int)(mv.x*scale);
	y1 = y + (int)(mv.y*scale);
	file.println(x0+" xunits "+y0+" yunits "+" moveto");
	file.println(x1+" xunits "+y1+" yunits "+" lineto");

	x0 = x1;
	y0 = y1;
      } else {
	x0 = x + (int)(mv.x*scale);
	y0 = y + (int)(mv.y*scale);

      }

    }
    file.println("stroke");
  }
}

