package graph;

import java.awt.*;
import java.applet.*;
import java.net.URL;
import java.util.*;
import java.io.*;

/*
**************************************************************************
**
**    Class PrintGraph
**
**************************************************************************
** Copyright (c) 1998
**      The Regents of the University of California.  All rights reserved.
**
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions
** are met:
** 1. Redistributions of source code must retain the above copyright
**    notice, this list of conditions and the following disclaimer.
** 2. Redistributions in binary form must reproduce the above copyright
**    notice, this list of conditions and the following disclaimer in the
**    documentation and/or other materials provided with the distribution.
** 3. All advertising materials mentioning features or use of this software
**    must display the following acknowledgement:
**      This product includes software developed by the University of
**      California, Berkeley and its contributors.
** 4. Neither the name of the University nor the names of its contributors
**    may be used to endorse or promote products derived from this software
**    without specific prior written permission.
**
** THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
** ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
** IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
** ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
** FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
** DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
** OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
** HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
** LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
** OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
** SUCH DAMAGE.
**
** Written by Mark Stemm (stemm@cs.berkeley.edu)
**************************************************************************
**
**    This class is an extension of G2Dint class.
**    It adds the ability to print the selected graph to a postscript file.
**
*************************************************************************/

/**
 *    This class is an extension of G2Dint class.
 *    It adds the ability to print the selected graph to a postscript file.
 *
 *    <h4>Additional KeyDown Events</h4>
 *     <dt>s
 *     <dd>Brings up dialog to print current graph to postscript file.
 *    </dl>
 *    <P>
 *
 * @version $Revision: 1.2 $, $Date: 1998/04/13 22:20:46 $.
 * @author Mark Stemm (stemm@cs.berkeley.edu)
 */

public class PrintGraph extends G2Dint {

  /*
*******************
** Public Methods
******************/

  /**
   * Pop up a dialog selecting file for output, and then print graph to file.
   */
  public void save(){
    FileDialogPopup fileDialogPopup;
    PrintStream file;
    
    // Open a file dialog for the output file
    fileDialogPopup = new FileDialogPopup("File For Graph Output:", FileDialog.SAVE);

    try {
      // Open the file for writing
      file=new PrintStream(new FileOutputStream(fileDialogPopup.getFile()));
      GraphPrinter pg = new GraphPrinter(this);
      pg.drawToFile(file);
    } catch (IOException e) {
      System.out.println("Couldn't write to file!");
    }
  }

  /**
   * This is identical to G2Dint's keyDown function with the addition
   * of a handler for the s (save to postscript file) key.
   */
  
  public boolean keyDown(Event e, int key) {
    
    if(xaxis==null || yaxis==null) return false;
    
    switch ( key ) {
    case 's':
      save();
      return true;
	
    default:
      //                             System.out.println("KeyPress "+e.key);
      return (super.keyDown(e, key));
    }
    
  }
}

  
