/*
 * Copyright (c) 1996-1998 Regents of the University of California.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the SPAND Research
 *      Group at the University of California Berkeley.
 * 4. Neither the name of the University nor of the Research Group may be
 *    used to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
/* $Header: /home/stemm/cvs/JavaSpand/Client.java,v 1.6 1998/06/26 18:41:20 stemm Exp $ */
import java.util.*;
import java.net.*;
import java.io.*;
import spand.*;

/**
 * This class implements a simple SPAND client that queries the Performance Server
    for information on a particular address. This client assumes that the Performance Server is localhost. <br>
    
    The usage is: <tt> client [-addr <addr>] [-perf | -raw |-summ] [-perfSvr <name/addr>]</tt> <br>
    <tt> [-addr <addr>] </tt> is the IP address (not name) for which to query the performance. <br>
    <tt> -perf </tt> means to get a Performance Response for that address. <br>
    <tt> -raw </tt> means to get all Performance Reports for that address. <br>
    <tt> -summ </tt> means to return the number of Performance Reports collected for that address.
    <tt> [-perfSvr <name/address>] </tt> is the IP address or name of the SPAND Performance Server.
 * @version $Revision: 1.6 $
 * @author SPAND Group, UC Berkeley.
 */
public class Client {
  public static void main(String argv[]) {
    InetAddress qaddr;
    Msg m=null, replyMsg=null;
    Requestor requestor;
    String perfSvr = new String("127.0.0.1");
    int i, argc=argv.length;
    String USAGESTR = new String("usage: client <addr> [-perf| -raw| -summ] [-perfSvr <name/addr>]");

    if (argc==0){
      System.err.println(USAGESTR);
      System.exit(-1);
    }
    
    try {
      qaddr = InetAddress.getByName(argv[0]);

      try {
	for(i=1; i<argc; i++){
	  if (argv[i].equals("-perf")){
	    m = new Msg("spand.GetPerf", new PerfRequestPayload(qaddr), true);
	  } else if (argv[i].equals("-raw")){
	    m = new Msg("spand.GetRaw", new RawRequestPayload(qaddr), true);
	  } else if (argv[i].equals("-summ")){
	    m = new Msg("spand.GetSumm", new SummRequestPayload(qaddr), true);
	  } else if (argv[i].equals("-perfSvr")){
	    if (i+1<argc && !argv[i+1].startsWith("-")){
	      perfSvr=argv[i+1];
	    } else {
	      System.err.println(USAGESTR);
	      System.exit(-1);
	    }
	  }
	}

	if (m==null){
	      System.err.println(USAGESTR);
	  System.exit(-1);
	}
	requestor = new Requestor(perfSvr);

	replyMsg=requestor.Request(m, true);
	System.err.println(replyMsg.toString());
      } catch (IOException e1){
	System.err.println("Couldn't connect to Perf Server "+perfSvr);
	System.exit(-1);
      }

    } catch (UnknownHostException e){
      System.err.println("Unknown host "+argv[0]);
      System.exit(-1);
    }
  }
}
