/*
 * Copyright (c) 1996-1998 Regents of the University of California.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the SPAND Research
 *      Group at the University of California Berkeley.
 * 4. Neither the name of the University nor of the Research Group may be
 *    used to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
/* $Header: /home/stemm/cvs/JavaSpand/spand/ArrayConv.java,v 1.7 1998/06/27 01:41:18 stemm Exp $ */
package spand;
import java.net.*;

/**
 * This object has several methods that are useful when converting
 * Java objects into byte streams.
 * @version $Revision: 1.7 $
 * @author SPAND Group, UC Berkeley.
 */

public class ArrayConv {

  /**
   * Read an int from the byte stream starting at offset <tt> offset
   * </tt> and return it, including conversion from network to host
   * order. The int is returned. */
  
    public static int ByteArrayToInt(byte array[], int offset) {

      int foo = (array[offset + 3] & 0xFF) | 
	((array[offset + 2] << 8) & 0xFF00) | 
	((array[offset + 1] << 16) & 0xFF0000) | 
	((array[offset] << 24) & 0xFF000000);

      offset +=4;
      return foo;
    }

  /**
   * Write the specified int to the byte array starting at offset
   * <tt>offset</tt>, including conversion from host to network
   * order.
   */
  
    public static void IntToByteArray(byte array[], int offset, int value) {
      
      array[offset + 3] = (byte) (value & 0xFF);
      array[offset + 2] = (byte)((value & 0xFF00) >> 8);
      array[offset + 1] = (byte)((value & 0xFF0000) >> 16);
      array[offset] = (byte)((value & 0xFF000000) >> 24);
      offset+=4;
    }

  /**
   * Read a short from the byte stream starting at offset <tt> offset
   * </tt> and return it, including conversion from network to host
   * order. The short is returned. */
  
    public static int ByteArrayToShort(byte array[], int offset) {

      int foo = ((array[offset + 1]) & 0xFF) | 
	((array[offset] << 8) & 0xFF00);
	
      offset +=2;
	
      return foo;
    }

  /**
   * Write the specified short to the byte array starting at offset
   * <tt>offset</tt>, including conversion from host to network
   * order. */
  
    public static void ShortToByteArray(byte array[], int offset, int value) {

      array[offset + 1] = (byte)((value & 0xFF));
      array[offset] = (byte)((value & 0xFF00) >> 8);

      offset +=2;
    }

  /**
   * Write the specified InetAddress object to the byte array starting
   * at offset <tt>offset</tt>, including conversion from host to
   * network order. */
  public static void AddrToByteArray(byte array[], int offset, InetAddress addr){
    
    byte addrBytes[] = addr.getAddress();
    for (int i=0, j=offset; i < 4; i++, j++){
      array[j]=addrBytes[i];
    }

    offset +=4;

  }

  /**
   * Read a InetAddress object from the byte stream starting at offset
   * <tt> offset </tt> and return it, including conversion from
   * network to host order. The new InetAddress object is returned. */
  
  public static InetAddress ByteArrayToAddr(byte array[], int offset){
    int addrBytes[] = new int[4];
    InetAddress addr = null;
    
    for (int i=0, j=offset; i < 4; i++, j++){
      if (array[j]<0){
	addrBytes[i] = array[j]+256;
      } else {
	addrBytes[i] = array[j];
      }
    }
    try {
      addr = InetAddress.getByName(addrBytes[0]+"."+addrBytes[1]+"."+addrBytes[2]+"."+addrBytes[3]);
    } catch (UnknownHostException e){
    }

    offset+=4;
    return addr;
  }

  /**
   * Write the specified InetAddress object to the byte array starting
   * at offset <tt>offset</tt>. */
  
  public static void StringToByteArray(byte array[], int offset, String s){
    /* Write the length of the string */
    ArrayConv.IntToByteArray(array, offset, s.length());

    offset+=4;
    
    /* Now write the string */
    byte[] bytes=s.getBytes();
    for (int i=0,j=offset; i<s.length(); i++, j++){
      array[j]=bytes[i];
    }
    offset+=s.length();

    /* To make sure things are word-aligned, write some additional bytes */
    for(int j=offset; j%4!=0; j++,offset++){
      array[j]=0;
    }
  }

  /**
   * Read a String object from the byte stream starting at offset <tt>
   * offset </tt> and return it. The new String object is returned. */
  
  public static String ByteArrayToString(byte array[], int offset){
    
    /* Read the length of the errorMsg string */
    int length = ArrayConv.ByteArrayToInt(array, offset);

    offset+=4;
    
    /* Now read the string */
    String s = new String(array, offset, length);

    offset+=length;
    return s;
  }
  
  /**
   * Write the specified boolean value to the byte array starting
   * at offset <tt>offset</tt>. */
  
  public static void BooleanToByteArray(byte array[], int offset, boolean b){
    if (b){
      IntToByteArray(array, offset, 1);
    } else {
      IntToByteArray(array, offset, 0);
    }
  }

  /**
   * Read a boolean value from the byte stream starting at offset <tt>
   * offset </tt> and return it. The boolean value is returned. */
  
  public static boolean ByteArrayToBoolean(byte array[], int offset){

    if (ByteArrayToInt(array, offset)==1){
      return true;
    } else {
      return false;
    }
  }
  

}
