/*
 * Copyright (c) 1996-1998 Regents of the University of California.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the SPAND Research
 *      Group at the University of California Berkeley.
 * 4. Neither the name of the University nor of the Research Group may be
 *    used to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
/* $Header: /home/stemm/cvs/JavaSpand/spand/PerfRequestPayload.java,v 1.7 1998/06/10 23:12:29 stemm Exp $ */
package spand;
import java.net.*;

/** A Performance Request Object, usually used with a
    "spand.GetPerf" handler.
 * @author SPAND Group, UC Berkeley.
 * @version $Revision: 1.7 $.
 */
public class PerfRequestPayload implements SendableSpandObject{

  /** The address of which to ask the performance */
  public InetAddress addr;

  /** The size of the Marshalled object. */
  final public int wfSize=4;

  /** Default Constructor */
  public PerfRequestPayload(){
  }

  /** Constructor that provides address of host to ask about */
  public PerfRequestPayload(InetAddress addr0)
       throws UnknownHostException {
	 addr = addr0;
  }

  /** Constructor that provides name of host to ask about */
  public PerfRequestPayload(String hostname) 
       throws UnknownHostException {
	 this(InetAddress.getByName(hostname));
  }

  /** Unpackage this object from a sequence of bytes. The number of
      bytes read is returned. */
  public int Unmarshal(byte[] wireData, int startIndex) {
    int pos=startIndex;
    addr = ArrayConv.ByteArrayToAddr(wireData, pos);
    pos+=4;

    return (pos-startIndex);
  }

  /** Package this object into a sequence of bytes. The number of
      bytes written is returned. If wireData is NULL, then no bytes
      are actually written, but the size of the marshalled object is
      returned. */
  public int Marshal(byte[] wireData, int startIndex) {
    if (wireData==null){
      return wfSize;
    }
    int pos=startIndex;
    ArrayConv.AddrToByteArray(wireData, pos, addr);
    pos+=4;
    
    return (pos-startIndex);
  }

  /** Return a String representation of this object. */
  public String toString(){
    return addr.toString();
  }
}
