/*
 * Copyright (c) 1996-1998 Regents of the University of California.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the SPAND Research
 *      Group at the University of California Berkeley.
 * 4. Neither the name of the University nor of the Research Group may be
 *    used to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
/* $Header: /home/stemm/cvs/JavaSpand/spand/ReportPayload.java,v 1.10 1998/06/03 20:26:11 stemm Exp $ */
package spand;

import java.net.*;
import java.io.*;

/**
 * The default performance report. Contains bandwidth and rtt estimates.
 * @version $Revision: 1.10 $
 * @author SPAND Group, UC Berkeley.
 */
public class ReportPayload implements SendableSpandObject{

  /** The source address of the Report */
  public InetAddress saddr;

  /** The source port of the Report (as an int) */
  public int sport;

  /** The destination address of the Report */
  public InetAddress daddr;

  /** The destination port of the Report (as an int) */
  public int dport;

  /** The time the report was made in seconds since the epoch */
  public int mostSigNTP;

  /** The time the report was made in microseconds sice the epoch */
  public int leastSigNTP;

  /** The number of bytes in this report */
  public int sampLen;

  /** The duration of the sample in milliseconds */
  public int sampDur;

  /** The number of packets in the sample */
  public int packetsReceived;

  /** A guess as to the number of packets lost in the sample */
  public int packetsLost;

  /** The average packet size for this sample */
  public int packetSize;

  /** The smoothed round trip time for this sample, in microseconds. */
  public int srtt;

  /** The size of the marshalled object */
  private int wfSize=48;

  /** Default constructor */
  public ReportPayload()
  {
  }

  /** Constructor that explicly sets fields. */
  public ReportPayload(InetAddress saddr0,
		       int sport0, InetAddress daddr0, int dport0,
		       int mostSigNTP0, int leastSigNTP0, int sampLen0, int sampDur0,
		       int packetsReceived0, int packetsLost0, int packetSize0, int srtt0)
       throws UnknownHostException 
  {
    saddr = saddr0;
    sport = sport0;
    daddr = daddr0;
    dport = dport0;
    mostSigNTP = mostSigNTP0;
    leastSigNTP = leastSigNTP0;
    sampLen = sampLen0;
    sampDur = sampDur0;
    packetsReceived = packetsReceived0;
    packetsLost = packetsLost0;
    packetSize = packetSize0;
    srtt = srtt0;
  }

  /** Unpackage this object from a sequence of bytes. The number of
      bytes read is returned. */
  public int Unmarshal(byte[] wireData, int startIndex)
  {
    int pos=startIndex;
    
    saddr = ArrayConv.ByteArrayToAddr(wireData, pos);
    pos+=4;
    sport = ArrayConv.ByteArrayToInt(wireData, pos);
    pos+=4;
    daddr = ArrayConv.ByteArrayToAddr(wireData, pos);
    pos+=4;
    dport = ArrayConv.ByteArrayToInt(wireData, pos);
    pos+=4;
    mostSigNTP = ArrayConv.ByteArrayToInt(wireData, pos);
    pos+=4;
    leastSigNTP = ArrayConv.ByteArrayToInt(wireData, pos);
    pos+=4;
    sampLen = ArrayConv.ByteArrayToInt(wireData, pos);
    pos+=4;
    sampDur = ArrayConv.ByteArrayToInt(wireData, pos);
    pos+=4;
    packetsReceived = ArrayConv.ByteArrayToInt(wireData, pos);
    pos+=4;
    packetsLost = ArrayConv.ByteArrayToInt(wireData, pos);
    pos+=4;
    packetSize = ArrayConv.ByteArrayToInt(wireData, pos);
    pos+=4;
    srtt = ArrayConv.ByteArrayToInt(wireData, pos);
    pos+=4;
    
    return (pos-startIndex);
  }

  /** Package this object into a sequence of bytes. The number of
      bytes written is returned. If wireData is NULL, then no bytes
      are actually written, but the size of the marshalled object is
      returned. */
  public int Marshal(byte[] wireData, int startIndex){

    if (wireData==null){
      return wfSize;
    }

    int pos=startIndex;
    
    ArrayConv.AddrToByteArray(wireData, pos, saddr);
    pos+=4;
    ArrayConv.IntToByteArray(wireData, pos, sport);
    pos+=4;
    ArrayConv.AddrToByteArray(wireData, pos, daddr);
    pos+=4;
    ArrayConv.IntToByteArray(wireData, pos, dport);
    pos+=4;
    ArrayConv.IntToByteArray(wireData, pos, mostSigNTP);
    pos+=4;
    ArrayConv.IntToByteArray(wireData, pos, leastSigNTP);
    pos+=4;
    ArrayConv.IntToByteArray(wireData, pos, sampLen);
    pos+=4;
    ArrayConv.IntToByteArray(wireData, pos, sampDur);
    pos+=4;
    ArrayConv.IntToByteArray(wireData, pos, packetsReceived);
    pos+=4;
    ArrayConv.IntToByteArray(wireData, pos, packetsLost);
    pos+=4;
    ArrayConv.IntToByteArray(wireData, pos, packetSize);
    pos+=4;
    ArrayConv.IntToByteArray(wireData, pos, srtt);
    pos+=4;
    
    return (pos-startIndex);
  }

  /** Return a String representation of this object. */
  public String toString() {
    return "saddr=" + saddr + " sport=" + sport +
      " daddr=" + daddr + " dport=" + dport +
      " mostSigNTP=" + mostSigNTP + " leastSigNTP=" + leastSigNTP +
      " sampLen=" + sampLen + " sampDur=" + sampDur +
      " packetsReceived=" + packetsReceived + " packetsLost=" + packetsLost +
      " packetSize=" + packetSize +
      " srtt=" + srtt;
  }
}
