/*
 * Copyright (c) 1996-1998 Regents of the University of California.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the SPAND Research
 *      Group at the University of California Berkeley.
 * 4. Neither the name of the University nor of the Research Group may be
 *    used to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
/* $Header: /home/stemm/cvs/JavaSpand/spand/Requestor.java,v 1.10 1998/06/03 20:26:11 stemm Exp $ */
package spand;
import java.net.*;
import java.io.*;
import java.util.*;

/**
 * A Requestor object sends a message to the perf server and waits
 * for a reply. It tries to contact the Perf Server using UDP and
 * falls back to TCP if the reply is too large to fit in a single
 * datagram or if there is no reponse using TCP.
 * @version $Revision: 1.10 $
 * @author SPAND Group, UC Berkeley.
 */
public class Requestor {

  /** Socket used for stream connection to server */
  public static Socket requesteeSock;
  
  /** Socket used for datagram connection to server */
  public static DatagramSocket reqDgramSock;

  /** Socket used for to stream data to server */
  static DataOutputStream outst;

  /** Socket used for to receive stream data from server */
  static DataInputStream inst;

   /** The name of the Perf Server */
  static String servername;

  /** Connect to the Perf Server using a Stream Socket */
  protected static void ConnectStreamSpand() throws IOException {
    requesteeSock = new Socket(servername, Spand.RequestPort);
    outst = new DataOutputStream(requesteeSock.getOutputStream());
    inst = new DataInputStream(requesteeSock.getInputStream());
  }

  /** Connect to the Perf Server using a Datagram Socket */
  protected static void ConnectDgramSpand() throws IOException {
    reqDgramSock = new DatagramSocket();
    reqDgramSock.setSoTimeout(1000);
  }

  /** Default constructor. It must take the name of Perf Server as an
     argument */
  public Requestor(String servername0) throws IOException{
    servername = servername0;
  }

  /** Close associated sockets when garbage collecting this object */
  protected void finalize() throws IOException {
    /* Close Sockets */
    if (requesteeSock!=null){
      requesteeSock.close();
    }
    if (reqDgramSock!=null){
      reqDgramSock.close();
    }
  };

  /** Make a request and receive a Message as a reply (if waitForReply
      is true) */
  public Msg Request(Msg m, boolean waitForReply)
       throws UnknownHostException, IOException
  {
    int size;

    /* Marshal the request message. If it won't fit in a single UDP
       datagram, use TCP instead */
    size=m.Marshal(null, 0);
    if (size>=Spand.MaxMsgSize){
      return RequestStream(m, waitForReply);
    } else {
      try {
	return RequestDgram(m, waitForReply);
      } catch (IOException e){
	return RequestStream(m, waitForReply);
      }
    }
  }

/** Send a Message to the Perf Server using a Datagram Socket */
  private Msg RequestDgram(Msg m, boolean waitForReply) throws IOException{
    int size;
    byte[] buf;
    DatagramPacket p;
    Msg replyMsg=null;

    if (reqDgramSock==null){
      ConnectDgramSpand();
    }
    
    size=m.Marshal(null, 0);
    buf = new byte[size];
    m.Marshal(buf, 0);
    p = new DatagramPacket(buf, size, InetAddress.getByName(servername), Spand.RequestPort);

    reqDgramSock.send(p);

    if (waitForReply){
      int numRetries=5;
      while (numRetries>0){
	try {
	  /* Wait for (timeout) seconds before retrying */
	  replyMsg = new Msg(reqDgramSock);
	  numRetries=0;
	} catch (InterruptedIOException e1){
	  reqDgramSock.send(p);
	  numRetries--;
	}
      }

      /* If error, use TCP instead */
      if (replyMsg==null || replyMsg.handler.equals("ErrorPayload")){
	return (RequestStream(m, waitForReply));
      }
      return replyMsg;
    } else {
      return null;
    }
  }

  /** Send a Message to the Perf Server using a Stream Socket */
  private Msg RequestStream(Msg m, boolean waitForReply) throws IOException{
    int size;
    byte[] buf;
    DatagramPacket p;
    Msg replyMsg;

    ConnectStreamSpand();
	
    size=m.Marshal(null, 0);
    buf = new byte[size];
    m.Marshal(buf, 0);
    outst.write(buf);

    if (waitForReply){
      replyMsg = new Msg(requesteeSock);
      return replyMsg;
    } else {
      return(null);
    }
  }
}







