/*
 * Copyright (c) 1996-1998 Regents of the University of California.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the SPAND Research
 *      Group at the University of California Berkeley.
 * 4. Neither the name of the University nor of the Research Group may be
 *    used to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
/* $Header: /home/stemm/cvs/JavaSpand/spand/SummResponsePayload.java,v 1.6 1998/06/03 20:26:12 stemm Exp $ */
package spand;

import java.net.*;


/**
 * A Summ. Response Message object contains the number of reports
 * received for a particular host.
 * @version $Revision: 1.6 $
 * @author SPAND Group, UC Berkeley.
 */
public class SummResponsePayload implements SendableSpandObject{

  /** The source addr that this message is about */
  public InetAddress addr;

  /** The number of reports seen for this address */
  public int numReports;

  /** The sizeof the marshalled object. */
  final public int wfSize=8;

  /** Default Constructor */
  public SummResponsePayload(){
  }

  /** Constructor from explicit fields */
  public SummResponsePayload(InetAddress addr,
			     int numReports)
  {
    this.addr = addr;
    this.numReports = numReports;
  }

  /** Constructor from explicit fields */
  public SummResponsePayload(String hostname, int numReportsp) throws UnknownHostException 
  {
    this(InetAddress.getByName(hostname), numReportsp);
  }

  /** Unpackage this object from a sequence of bytes. The number of
      bytes read is returned. */
  public int Unmarshal(byte[] wireFormat, int startIndex) {

    int pos=startIndex;
    
    addr = ArrayConv.ByteArrayToAddr(wireFormat, pos);
    pos+=4;
    numReports = ArrayConv.ByteArrayToInt(wireFormat, pos);
    pos+=4;

    return (pos-startIndex);
  }

  /** Package this object into a sequence of bytes. The number of
      bytes written is returned. If wireData is NULL, then no bytes
      are actually written, but the size of the marshalled object is
      returned. */
  public int Marshal(byte[] wireFormat, int startIndex) {
    if (wireFormat==null){
      return wfSize;
    }
    int pos=startIndex;
	
    ArrayConv.AddrToByteArray(wireFormat, pos, addr);
    pos+=4;
    ArrayConv.IntToByteArray(wireFormat, pos, numReports);
    pos+=4;
    
    return (pos-startIndex);
  }

  /** Return a String representation of this object. */
  public String toString() {
    return "addr=" + addr.toString() + " numReports=" + numReports;
  }
}
