/*
 * Copyright (c) 1996-1998 Regents of the University of California.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the SPAND Research
 *      Group at the University of California Berkeley.
 * 4. Neither the name of the University nor of the Research Group may be
 *    used to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
/* $Header: /home/stemm/cvs/SPAND/javasrc/Snapshot/SnapshotUIModifier.java,v 1.6 1998/06/25 23:22:38 stemm Exp $ */

import java.awt.*;
import java.io.*;
import spand.*;
import java.util.*;
import java.net.*;

/**
 * This is a thread that loops forever, periodically contacting the
 * Performance Server for a new summary of the database and updating
 * the scrolling list.
 * @version $Revision: 1.6 $
 * @author SPAND Group, UC Berkeley.
 */

public class SnapshotUIModifier extends Thread {
  
  List list;
  Label reportCountLabel;
  Checkbox update;
  String spandSvr;
  
  /** Default constructor. Takes scrolling list as argument */
  public SnapshotUIModifier(List list, Label reportCountLabel, Checkbox update, String spandSvr){
    this.list=list;
    this.reportCountLabel=reportCountLabel;
    this.update=update;
    this.spandSvr=spandSvr;
  }

  /** Add an entry to the scrolling list for a new host */
  protected void AddItem(List list, String string, int numReports)
  {
    int position=0;
    String name;
    Integer i = new Integer(0);

    while (position<list.countItems()){
      // Get the string from the scrolling list and extract the number of reports
      name=list.getItem(position);
      StringTokenizer tok = new StringTokenizer(name);
      tok.nextToken();
      String count = tok.nextToken();
      if (i.parseInt(count)<numReports){
	list.addItem(string+" "+numReports, position);
	return;
      }
      position++;
    }
    list.addItem(string+" "+numReports, position);

  }

  /** Start the thread. */
  public void run() {
    int position;
    SummRequestPayload payload;
    Requestor requestor;
    int reportCount=0;
    
    // Loop forever, getting reports and adding/modifying
    // entries in the scrolling list.
    while(true){
      
      // Sleep for 5 seconds
      try {sleep(5000);} catch(InterruptedException e){};

      if (update.getState()){
	synchronized(list){
	  try {
	    requestor =  new Requestor(spandSvr);
	    InetAddress qaddr = InetAddress.getByName("0.0.0.0");
	    
	    // Ask for a summary from the performance server
	    try {
	      Msg replyMsg = requestor.Request(new Msg("spand.GetSumm",
						       new SummRequestPayload(qaddr), true), true);
	      Enumeration e = ((SendableSpandObjectColl) replyMsg.toObject()).objects.elements();
	      
	      // Remove all the items from the scrolling list
	      if (list.countItems()>0){
		list.delItems(0, list.countItems()-1);
	      }
	      // Loop over Responses, adding addr/count pairs to scrolling list
	      reportCount=0;
	      while (e.hasMoreElements()){
		SummResponsePayload s = (SummResponsePayload) e.nextElement();
		AddItem(list, s.addr.getHostAddress(), s.numReports);
		reportCount+=s.numReports;
	      }
	      reportCountLabel.setText(reportCount+" Reports in Performance Server's database.");
	    } catch (IOException e){
	      System.err.println("Problem getting Summary");
	    }
	  } catch (IOException e){
	    System.out.println("Couldn't make Spand Requestor");
	  }
	}
      }
    }
  }
}
  

