/*
 * Copyright (c) 1996-1998 Regents of the University of California.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the SPAND Research
 *      Group at the University of California Berkeley.
 * 4. Neither the name of the University nor of the Research Group may be
 *    used to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
/* $Header: /home/stemm/cvs/SPAND/javasrc/SpandProxy/spandFilters/Lights.java,v 1.3 1998/06/25 23:23:21 stemm Exp $ */

package spandFilters;

import org.doit.muffin.*;
import java.util.Hashtable;

/**
 * The Lights Muffin filter is used to indicate the web object download times to distant network hosts. When Muffin is used and the Lights filter is enabled, small lights are inserted next to hyperlinks that give a rough idea of the time it would take to retrieve the object listed in the hyperlink. If a user clicks on the image, the user downloads a Java applet that can be used to obtain more information about the performance to that host. The filter does this by constructing special hyperlinks that point to image and class files that the Filter serves out when requested. The color of the image is determined by querying the performance from a SPAND Performance Server. The Filter has the following configurable options:
 <lu>
 <li> <tt> spandObjectHost </tt>: The name that this filter will insert in the special hyperlinks to images. The only requirement is that the combination of name and port should not map onto any real network host and port.
 <li> <tt> spandObjectPort </tt>: The port that this filter will insert in the special hyperlinks to images. The only requirement is that the combination of name and port should not map onto any real network host and port.
 <li> <tt> spandSvr </tt>: The name of the local domain's SPAND Performance Server.
 <li> <tt> lightsOn </tt>: If selected, then lights are inserted. If not, then the html is unmodified.
 <li> <tt> greenThreshold </tt>: Any hyperlink with a bandwidth greater than <tt> greenThreshold </tt> kilobits/sec will be marked with a green light.
 <li> <tt> redThreshold </tt>: Any hyperlink with a bandwidth less than <tt> redThreshold </tt> kilobits/sec will be marked with a green light.
 <li> <tt> imageDir </tt>: The directory that contains the image files to serve out.
 <li> <tt> classDir </tt>: The directory that contains the .class files for the Java applet.
 </lu>
 * @version $Revision: 1.3 $
 * @author SPAND Group, UC Berkeley.
 */


public class Lights implements FilterFactory {
    FilterManager manager;
    Prefs prefs;
    LightsFrame frame = null;
  
    public Lights() {
    }

    public void setManager (FilterManager manager) {
	this.manager = manager;
    }
    
    public void setPrefs (Prefs prefs) {
	this.prefs = prefs;

	boolean o = prefs.getOverride ();
	prefs.setOverride (false);
	prefs.putString("Lights.spandObjectHost", "illusion");
	prefs.putInteger("Lights.spandObjectPort", 61069);
	prefs.putString("Lights.spandSvr", "localhost");
	prefs.putBoolean("Lights.lightsOn", true);
	prefs.putInteger("Lights.greenThreshold", 50);
	prefs.putInteger("Lights.redThreshold", 5);
	prefs.putString("Lights.imageDir", "images/");
	prefs.putString("Lights.classDir", "class/");
	prefs.setOverride (o);
    }

    public Prefs getPrefs () {
	return prefs;
    }

    public void viewPrefs () {
	if (frame == null) {
	    frame = new LightsFrame (prefs, this);
	}
	frame.setVisible (true);
    }
    
    public Filter createFilter () {
	Filter f = new LightsFilter (this);
	f.setPrefs (prefs);
	return f;
    }

  public void shutdown (){
  }
      
  
    void save () {
	manager.save (this);
    }

    void process (String s) {
	if (frame != null) {
	    frame.text.append (s);
	}
    }
}

