/*
 * Copyright (c) 1996-1998 Regents of the University of California.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the SPAND Research
 *      Group at the University of California Berkeley.
 * 4. Neither the name of the University nor of the Research Group may be
 *    used to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
/* $Header: /home/stemm/cvs/SPAND/javasrc/SpandProxy/spandFilters/LightsFilter.java,v 1.3 1998/06/24 18:12:56 stemm Exp $ */
/* LightsFilter.java -- */

package spandFilters;

import java.net.*;
import java.io.*;
import org.doit.muffin.*;
import org.doit.io.*;
import org.doit.html.*;
import java.util.*;
import spand.*;

public class LightsFilter implements HttpFilter, ContentFilter {
  Lights factory;
  Prefs prefs;
  Request request = null;
  InputObjectStream in = null;
  OutputObjectStream out = null;
  Url curRef;
  
  LightsFilter (Lights factory) {
    this.factory = factory;
  }
  
  public void setPrefs (Prefs prefs) {
    this.prefs = prefs;
  }
  
  public boolean wantRequest(Request request){
    return request.getURL().startsWith("http://"+prefs.getString("Lights.spandObjectHost")+":"+prefs.getString("Lights.spandObjectPort"));
    
  }
  
  public void sendRequest(Request request){
    
  }
  
  public Reply recvReply(Request request){
    Reply reply = new Reply ();
    
    reply.setStatusLine("HTTP/1.0 200 Ok");
    reply.setHeaderField ("Content-type", "text/html");
    
    if (request.getPath ().startsWith ("/images/") ||
	request.getPath().startsWith("/class/") ||
	request.getPath().startsWith("/host/marker.txt"))
      {
	String path=new String();
	
	if (request.getPath().startsWith("/images/")){
	  path = new String(prefs.getString("Lights.imageDir")+request.getPath().substring(8));
	    
	  if (request.getPath ().endsWith("gif")) 
	    reply.setHeaderField ("Content-type", "image/gif");
	  else
	    reply.setHeaderField ("Content-type", "image/jpeg");
	  
	} else if (request.getPath ().startsWith ("/class/")){
	  path = new String(prefs.getString("Lights.classDir")+request.getPath().substring(7));
	  reply.setHeaderField ("Content-type", "application/java-vm");
	} else if (request.getPath ().startsWith ("/host/marker.txt")){
	  path=new String("marker.txt");
	  reply.setHeaderField ("Content-type", "text/plain");
	}
      
	try{
	  InputStream content=null;
	  content = new FileInputStream(path);
	  reply.setContent(content);
	} catch (Exception e) {
	}
      }
    else if (request.getPath ().startsWith ("/host/"))
      {
	
	StringBuffer html = new StringBuffer();
	
	String reqHost = request.getPath ().substring (6);
	if (isHostname(reqHost)) {
	  try {
	    InetAddress initialHostA = InetAddress.getByName(reqHost);
	    reqHost = initialHostA.getHostAddress();
	  }
	  catch (UnknownHostException ue) {
	    System.err.println("Cannot find host " + reqHost);
	    return null;
	  }
	}
	
	String localhost;
	
	html.append("<hr><applet code=Snapshot.class width=800 height=600"+
		    " codebase=http://" + 
		    prefs.getString("Lights.spandObjectHost") + ":"+ prefs.getString("Lights.spandObjectPort") +
		    "/class/> <param name=markers value=marker.txt> " +
		    "<param name=host value=" + reqHost + ">" +
		    "<param name=spand_server value=" + prefs.getString("Lights.spandSvr") + "> </applet>\n");
	byte buf[] = (html.toString()).getBytes ();
	reply.setHeaderField ("Content-length", 
			      Integer.toString (buf.length));
	reply.setContent((InputStream) new ByteArrayInputStream (buf));
	
      }
    else
      {
	// what should we serve here??? XXXXX
      }
    
    return reply;
  }    
  
  private static boolean isHostname(String s) {
    
    char[] ca = s.toCharArray();
    // if we see a character that is neither a digit nor a period
    // then s is probably a hostname
    for (int i = 0; i < ca.length; i++) {
      if (!Character.isDigit(ca[i])) {
	if (ca[i] != '.') {
	  return true;
	}
      }
    }
    return false;
  }
  
  public void close(){
    
  }
  
  public void setInputObjectStream (InputObjectStream in) {
    this.in = in;
  }
  
  public void setOutputObjectStream (OutputObjectStream out) {
    this.out = out;
  }
  
  public boolean needsFiltration (Request request, Reply reply) {
    this.request = request;
    String s = reply.getHeaderField ("Content-type");
    return s != null && s.startsWith ("text/html");
  }
  
  public void run () {
    try {
      boolean addingLight = false;
      boolean lightsOn = prefs.getBoolean ("Lights.lightsOn");
      int greenT = prefs.getInteger ("Lights.greenThreshold");
      int redT = prefs.getInteger("Lights.redThreshold");
      int objectPort = prefs.getInteger("Lights.spandObjectPort");
      String objectHost = prefs.getString("Lights.spandObjectHost");
      String perfSvr = prefs.getString("Lights.spandSvr");
      
      
      Object obj;
      Tag tag;
      
      
      while ((obj = in.read())!=null){
	Token token = (Token) obj;
	
	if (token.getType()==Token.TT_TAG && lightsOn){
	  tag = token.createTag ();
	  out.write(token);
	  if (addingLight && tag.is ("/a")) {
	    int perf = -1;
	    
	    Requestor requestor=new Requestor(perfSvr);
	    Msg m = new Msg("spand.GetPerf", new PerfRequestPayload(curRef.GetHostname()), true);
	    
	    Msg replyMsg =requestor.Request(m, true);
	    if (replyMsg.handler.equals("spand.PerfResponsePayload")){
	      PerfResponsePayload perfResp = (PerfResponsePayload) replyMsg.toObject();
	      perf = perfResp.availBandwidth/1000;
	    }
	    
	    String s;
	    s = "<a href=http://" + objectHost + ":" + 
	      (objectPort) + "/host/" + curRef.GetHostname()+">";
	    if (perf == -1)
	      s += "<img src=http://" + objectHost + ":" + 
		objectPort + 
		"/images/SPAND_question.gif></a>";
	    else if (perf > greenT)
	      s += "<img src=http://" + objectHost + ":" + 
		objectPort + 
		"/images/SPAND_good.gif></a>";
	    else if (perf < redT)
	      s += "<img src=http://" + objectHost + ":" + 
		objectPort + 
		"/images/SPAND_bad.gif></a>";
	    else
	      s += "<img src=http://" + objectHost + ":" + 
		objectPort + 
		"/images/SPAND_medium.gif></a>";
	    //			out.write(s.getBytes(), 0, s.length());
	    out.write(new ByteArray(s));
	    continue;
	  }
	  
	  if (tag.has ("href")) {
	    addingLight = true;
	    String s = tag.get("href");
	    curRef = new Url(s, request.getURL());
	    /* factory.process ("adding light for " + 
	       curRef.GetHostname() +
	       " to " + request.getURL() + "\n");
	    */
	  }
	}
	else 
	  out.write(token);
      }
      
      out.flush ();
      out.close ();
    }
    catch (Exception e)
      {
	e.printStackTrace ();
      }
  }
  
}
