/*
 * Copyright (c) 1996-1998 Regents of the University of California.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the SPAND Research
 *      Group at the University of California Berkeley.
 * 4. Neither the name of the University nor of the Research Group may be
 *    used to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
/* $Header: /home/stemm/cvs/SPAND/javasrc/SpandProxy/spandFilters/LightsFrame.java,v 1.2 1998/06/24 18:12:56 stemm Exp $ */
/* LightsFrame.java */

package spandFilters;

import java.awt.*;
import java.awt.event.*;
import org.doit.muffin.*;

public class LightsFrame extends Frame implements ActionListener, WindowListener
{
    Prefs prefs;
    Lights parent;
    Checkbox lightsOn;
    TextArea text;
    TextField spandObjectHost, spandObjectPort;
    TextField greenThreshold, redThreshold;
    TextField imageDir, classDir, spandSvr;
  
    public LightsFrame (Prefs prefs, Lights parent)
    {
	super ("Lights");

	this.prefs = prefs;
	this.parent = parent;

	setBackground (Color.lightGray);
	setFont (new Font ("Helvetica", Font.BOLD, 12));

	Panel panel = new Panel ();
	GridBagLayout layout = new GridBagLayout ();
	panel.setLayout (layout);
	GridBagConstraints c;

	lightsOn = new Checkbox ("SpandLights");
	lightsOn.setState (prefs.getBoolean ("Lights.lightsOn"));
	c = new GridBagConstraints ();
	c.gridwidth = GridBagConstraints.REMAINDER;
	layout.setConstraints (lightsOn, c);
	panel.add (lightsOn);

	Label greenLabel = new Label ("Green Light Threshold:", Label.RIGHT);
	c = new GridBagConstraints ();
	c.anchor = GridBagConstraints.WEST;
	layout.setConstraints (greenLabel, c);
	panel.add (greenLabel);

	greenThreshold = new TextField (2);
	greenThreshold.setText (prefs.getString ("Lights.greenThreshold"));
	c = new GridBagConstraints ();
	c.anchor = GridBagConstraints.WEST;
	layout.setConstraints (greenThreshold, c);
	panel.add (greenThreshold);

	Label redLabel = new Label ("Red Light Threshold:", Label.RIGHT);
	c = new GridBagConstraints ();
	c.anchor = GridBagConstraints.WEST;
	layout.setConstraints (redLabel, c);
	panel.add (redLabel);

	redThreshold = new TextField (2);
	redThreshold.setText (prefs.getString ("Lights.redThreshold"));
	c = new GridBagConstraints ();
	c.gridwidth = GridBagConstraints.REMAINDER;
	c.anchor = GridBagConstraints.WEST;
	layout.setConstraints (redThreshold, c);
	panel.add (redThreshold);

	Label spandSvrLabel = 
	    new Label ("Spand Perf. Server:", Label.RIGHT);
	c = new GridBagConstraints ();
	c.anchor = GridBagConstraints.WEST;
	layout.setConstraints (spandSvrLabel, c);
	panel.add (spandSvrLabel);

	spandSvr = new TextField (20);
	spandSvr.setText(prefs.getString("Lights.spandSvr"));

	c = new GridBagConstraints ();
	c.gridwidth = GridBagConstraints.REMAINDER;
	c.anchor = GridBagConstraints.WEST;
	layout.setConstraints (spandSvr, c);
	panel.add (spandSvr);
	
	Label spandObjectHostLabel = 
	    new Label ("SpandObject Server Host:", Label.RIGHT);
	c = new GridBagConstraints ();
	c.anchor = GridBagConstraints.WEST;
	layout.setConstraints (spandObjectHostLabel, c);
	panel.add (spandObjectHostLabel);

	spandObjectHost = new TextField (20);
	spandObjectHost.setText(prefs.getString("Lights.spandObjectHost"));

	c = new GridBagConstraints ();
	c.gridwidth = GridBagConstraints.REMAINDER;
	c.anchor = GridBagConstraints.WEST;
	layout.setConstraints (spandObjectHost, c);
	panel.add (spandObjectHost);

	Label spandObjectPortLabel = 
	    new Label ("SpandObject Server Port:", Label.RIGHT);
	c = new GridBagConstraints ();
	c.anchor = GridBagConstraints.WEST;
	layout.setConstraints (spandObjectPortLabel, c);
	panel.add (spandObjectPortLabel);

	spandObjectPort = new TextField (5);
	spandObjectPort.setText(prefs.getString("Lights.spandObjectPort"));

	c = new GridBagConstraints ();
	c.gridwidth = GridBagConstraints.REMAINDER;
	c.anchor = GridBagConstraints.WEST;
	layout.setConstraints (spandObjectPort, c);
	panel.add (spandObjectPort);

	Label imageDirLabel = 
	    new Label ("Directory where image files reside:", Label.RIGHT);
	c = new GridBagConstraints ();
	c.anchor = GridBagConstraints.WEST;
	layout.setConstraints (imageDirLabel, c);
	panel.add (imageDirLabel);

	imageDir = new TextField (20);
	imageDir.setText(prefs.getString("Lights.imageDir"));

	c = new GridBagConstraints ();
	c.gridwidth = GridBagConstraints.REMAINDER;
	c.anchor = GridBagConstraints.WEST;
	layout.setConstraints (imageDir, c);
	panel.add (imageDir);

		Label classDirLabel = 
	    new Label ("Directory where class files reside:", Label.RIGHT);
	c = new GridBagConstraints ();
	c.anchor = GridBagConstraints.WEST;
	layout.setConstraints (classDirLabel, c);
	panel.add (classDirLabel);

	classDir = new TextField (20);
	classDir.setText(prefs.getString("Lights.classDir"));

	c = new GridBagConstraints ();
	c.gridwidth = GridBagConstraints.REMAINDER;
	c.anchor = GridBagConstraints.WEST;
	layout.setConstraints (classDir, c);
	panel.add (classDir);

	text = new TextArea ();
	text.setEditable (false);

	c = new GridBagConstraints ();
	c.gridwidth = GridBagConstraints.REMAINDER;
	layout.setConstraints (text, c);
	panel.add (text);
	
	add ("Center", panel);

	Button b;
	Panel buttonPanel = new Panel ();
	buttonPanel.setLayout (new GridLayout (1, 4));
	b = new Button ("Apply");
	b.setActionCommand ("doApply");
	b.addActionListener (this);
	buttonPanel.add (b);
	b = new Button ("Save");
	b.setActionCommand ("doSave");
	b.addActionListener (this);
	buttonPanel.add (b);
	b = new Button ("Clear");
	b.setActionCommand ("doClear");
	b.addActionListener (this);
	buttonPanel.add (b);
	b = new Button ("Close");
	b.setActionCommand ("doClose");
	b.addActionListener (this);
	buttonPanel.add (b);

	add ("South", buttonPanel);

	addWindowListener (this);
	
	pack ();
	setSize (getPreferredSize ());

	show ();
    }

    public void actionPerformed (ActionEvent event)
    {
	String arg = event.getActionCommand ();
	
	if ("doApply".equals (arg))
	{
	    prefs.putString ("Lights.spandObjectHost",
			     spandObjectHost.getText());
	    Integer foo = new Integer(spandObjectPort.getText());
	    prefs.putInteger ("Lights.spandObjectPort", foo.intValue());
	    prefs.putString("Lights.spandSvr", spandSvr.getText());
	    prefs.putBoolean("Lights.lightsOn", lightsOn.getState());
	    foo = new Integer(greenThreshold.getText());
	    prefs.putInteger("Lights.greenThreshold", foo.intValue());
	    foo = new Integer(redThreshold.getText());
	    prefs.putInteger("Lights.redThreshold", foo.intValue());
	    prefs.putString("Lights.imageDir", imageDir.getText());
	    prefs.putString("Lights.classDir", classDir.getText());
	}
	else if ("doSave".equals (arg))
	{
	    parent.save ();
	}
	else if ("doClose".equals (arg))
	{
	    setVisible (false);
	}
	else if ("doClear".equals (arg))
	{
	    text.setText ("");
	}
    }

    public void windowActivated (WindowEvent e)
    {
    }
  
    public void windowDeactivated (WindowEvent e)
    {
    }
  
    public void windowClosing (WindowEvent e)
    {
	setVisible (false);
    }
  
    public void windowClosed (WindowEvent e)
    {
    }
  
    public void windowIconified (WindowEvent e)
    {
    }
  
    public void windowDeiconified (WindowEvent e)
    {
    }
  
    public void windowOpened (WindowEvent e)
    {
    }
}
