/*
 * Copyright (c) 1996-1998 Regents of the University of California.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the SPAND Research
 *      Group at the University of California Berkeley.
 * 4. Neither the name of the University nor of the Research Group may be
 *    used to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
/* $Header: /home/stemm/cvs/Socket++/Ber.cc,v 1.2 1998/05/28 20:15:29 stemm Exp $ */
static float Sock_Random(void)
{
     float  temp;
#ifdef RANDOM
     while ((temp = (double) (random() % 10000000) / 10000000) == 0.0);
#else
#ifdef RAND
     while ((temp = (double) (rand() % 10000000) / 10000000) == 0.0);
#endif
#endif
     return (temp);
}


/* Return a Poisson variate with specified mean */
static int Sock_PoissonDist(float err_prob)
{
     double   prob;

     prob = -err_prob * log(Sock_Random());
     return (int) (1 / (8.0 * prob));
}

/*
 * Poisson error generator 
 */

void Sock_PoissonError(Socket *sdPtr, char *pktPtr, int size)
{
     static int first_time = 1;
     int num_errors = 0;
     
     while (sdPtr->berState.bytesToNextError <= size) {
	  if (first_time)
	       first_time = 0;
	  else {
	       pktPtr[sdPtr->berState.bytesToNextError] = 
		    pktPtr[sdPtr->berState.bytesToNextError] ^ 
			 (1 << (int)(Sock_Random() * 8));
	       sdPtr->berState.bitsDropped++;
	  }
          sdPtr->berState.bytesToNextError += 
	       Sock_PoissonDist(sdPtr->berState.errorProb[0]);
     }
     sdPtr->berState.bytesToNextError -= size;
     sdPtr->berState.totalBits += size <<3;
}

void Socket::SetBer(Socket *sdPtr, float val)
{
     sdPtr->berState.ErrorGen = Sock_PoissonError;
     sdPtr->berState.errorProb[0] = val;
     sdPtr->berState.bytesToNextError = 0;
     sdPtr->berState.bitsDropped = 0;
     sdPtr->berState.totalBits = 0;
}

float Socket::GetBer(Socket *sdPtr) 
{
     return(sdPtr->berState.errorProb[0]);
}

