/*
 * Copyright (c) 1996-1998 Regents of the University of California.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the SPAND Research
 *      Group at the University of California Berkeley.
 * 4. Neither the name of the University nor of the Research Group may be
 *    used to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
/* $Header: /home/stemm/cvs/Socket++/Raw.cc,v 1.4 1998/05/28 20:15:30 stemm Exp $ */
#include <stdio.h>
#include <string.h>
#include <strings.h>
#include <stdlib.h>
#include <signal.h>
#include <fcntl.h>
#include <errno.h>
#include <assert.h>
#include <sys/file.h>
#include <sys/stat.h>
#include <math.h>
#include "porting.h"
#include "dbgprintlib.h"
#include "Socket.h"
#include "Raw.h"

RawSocket::RawSocket(int isCtrl = SOCK_NOT_CTRL) : Socket(isCtrl)
{
    if (InitSocket() != SOCK_SUCCESS) {
	state_ = SOCK_STATE_CLOSED;
	return;
    }
}
    
RawSocket::RawSocket() : Socket(SOCK_NOT_CTRL)
{
    if (InitSocket() != SOCK_SUCCESS) {
	state_ = SOCK_STATE_CLOSED;
	return;
    }
}

Sock_Status
RawSocket::InitSocket()
{
    struct protoent *proto;
    int i=1, j=sizeof(i);
    char *protoStr;
#ifdef THREADED
    char tempstr[256];
    struct protoent protoTemp;
#endif

    protocol_ = SOCK_IP;
    type_ = SOCK_RAW;

    protoStr="ip";
#ifdef THREADED
    proto = &protoTemp;
    proto = getprotobyname_r(protoStr, proto, tempstr, 256);
#else
    proto = getprotobyname(protoStr);
#endif

    /* get the socket */
    if ((rfd_ = sfd_ = socket(AF_INET, type_, proto->p_proto)) == -1) {
	debug_printf(110, "RawSock_socket: ERR opening socket");
	endprotoent();
	return SOCK_FAILURE;
    }
    endprotoent();
    SetBufferSize(SOCK_DEFAULT_BUF_SIZE);
    if (setsockopt(sfd_, IPPROTO_IP, IP_HDRINCL, (char *)&i,
		   sizeof(i)) < 0) {
	perror("traceroute: IP_HDRINCL");
	exit(6);
    }

    mtu_ = 1500;
    return SOCK_SUCCESS;
}

int 
RawSocket::Connect(char *hostname, Port port)
{
    Socket::Connect(hostname, port);
    
    state_ = SOCK_STATE_CONNECTED;
    return(SOCK_SUCCESS);

}

int 
RawSocket::Listen(Port port)
{
    int temp;
    
    state_ = SOCK_STATE_LISTENING;
    return(SOCK_SUCCESS);
}

RawSocket *
RawSocket::Accept()
{
    int temp;
    RawSocket *cdPtr;

    cdPtr = new RawSocket(isCtrl_);
    
    *cdPtr = *this;
    temp = cdPtr->AcceptOW();
    if (temp != SOCK_SUCCESS) {
	delete cdPtr;
	cdPtr = NULL;
    } else {
	rfd_ = -1;
	sfd_ = -1;
    }
    return(cdPtr);
}

Sock_Status
RawSocket::AcceptOW()
{

    debug_printf(112, "RawSock_Accept\n");
    Socket::AcceptOW();

    side_ = SOCK_SIDE_SERVER;
    state_ = SOCK_STATE_ACCEPTED;
    return(SOCK_SUCCESS);
}

int 
RawSocket::Recv(char *buffer, int length)
{
    int status;

    debug_printf(112, "RawSock_Recv %d\n", length);
    if ((status = WaitForPktRecv()) != SOCK_SUCCESS)
	return status;
    status = RecvDgram(buffer, length);
    return status;
}

int
RawSocket::RecvFull(char *buffer, int length)
{
    int status;

    debug_printf(112, "RawSock_Recv %d\n", length);
    if ((status = WaitForPktRecv()) != SOCK_SUCCESS)
	return status;
    status = RecvDgramFull(buffer, length);
    return status;
}

int 
RawSocket::Send(char *buffer, int length)
{
    int status;

    debug_printf(112, "RawSock_Send %d\n", length);
    WaitForPktSend(length);
    status = SendDgram(buffer, length);
    if (status < 0) 
	lastPktSize_ = 0;
    return (status);
}

