/*
 * Copyright (c) 1996-1998 Regents of the University of California.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the SPAND Research
 *      Group at the University of California Berkeley.
 * 4. Neither the name of the University nor of the Research Group may be
 *    used to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
/* $Header: /home/stemm/cvs/Socket++/SocketTypes.h,v 1.2 1998/05/28 20:15:30 stemm Exp $ */
#ifndef SS_SOCKET_TYPES
#define SS_SOCKET_TYPES

#include "porting.h"

#define SOCK_RELIABLE_DGRAM             10
#define SOCK_NO_TIMEOUT                 -1
#define SOCK_ANY_PORT                   0
#define SOCK_SUCCESS                    0
#define SOCK_FAILURE                    -1
#define SOCK_TIMEOUT                    -2
#define SOCK_FD_CLOSED                  -3

#define SOCK_MAX_MULTICAST_HOSTS                4
#define SOCK_MAX_HOST_NAME_LEN                  80
#define SOCK_MAX_IP_LEN                  	16
#define SOCK_MAX_HANDOFF_HOSTS                  4
#define SOCK_MAX_CTRL_HOSTS                     2

#define SOCK_SIDE_CLIENT                        1<<1
#define SOCK_SIDE_SERVER                        1<<2

#define SOCK_CTRL_TIMEOUT_SEC                   0
#define SOCK_CTRL_TIMEOUT_USEC                  500000
#define SOCK_MIN_USLEEP                         1500
#include <stdio.h>
#include <fcntl.h>
#include <netdb.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <netinet/in.h>
#ifdef BSDI
#define TCP_NODELAY     0x01    /* don't delay send to coalesce packets */
#define	TCP_MAXSEG	0x02	/* set maximum segment size */
#else
#include <netinet/tcp.h>
#endif
#include <netinet/in_systm.h>
#include <netinet/ip.h>
#include <arpa/inet.h>
#include <sys/time.h>

#define SOCK_DEFAULT_BUF_SIZE                   32768
#define SOCK_MAXHOSTNAME                        1024

#define SOCK_NOT_BROADCAST              0
#define SOCK_BROADCAST                  1
#define SOCK_DONT_USE_CTRL            	0
#define SOCK_USE_CTRL                   1
#define SOCK_NOT_CTRL                   0
#define SOCK_CTRL                       1
#define SOCK_COMPATIBLE                 1
#define SOCK_NOT_COMPATIBLE             0
#define SOCK_BLOCK                      1
#define SOCK_NON_BLOCK                  0

#define SOCK_STATE_ALLOCATED                    1
#define SOCK_STATE_LISTENING                    2
#define SOCK_STATE_CONNECTED                    3
#define SOCK_STATE_ACCEPTED                     4
#define SOCK_STATE_HANDOFF                      5
#define SOCK_STATE_CLOSED                       6
#define SOCK_STATE_CLOSING                      7

#define SOCK_CTRL_SET_RATE          	 3
#define SOCK_CTRL_SET_REMOTE         	 4
#define SOCK_CTRL_SET_REMOTE_CTRL    	 5
#define SOCK_CTRL_SET_REMOTE_CTRL_PORT 	 6
#define SOCK_CTRL_NEW_CTRL_CHANNEL     	 7
#define SOCK_CTRL_FAREWELL               8
#define SOCK_CTRL_CLOSE_CTRL_CHANNEL  	 9
#define SOCK_CTRL_HELLO                 10
#define SOCK_CTRL_ACK			11 /* A Socket-library-layer ack */

#define SOCK_UDP			1
#define SOCK_UDP_RELIABLE		2
#define SOCK_UDP_MULTI			3
#define SOCK_UDP_REL_MULTI		4
#define SOCK_TCP			5
#define SOCK_IP				6
#define SOCK_UDP_CTRL			7
#define SOCK_UDP_MULTI_CTRL		8
#define SOCK_TCP_CTRL			9
#define SOCK_ANY			10

typedef int Port;
typedef int Usec;
typedef struct sockaddr_in Address;
typedef int Sock_Status;
typedef struct timeval timev;

typedef struct SockCtrlHello {
     char ipAddr[SOCK_MAX_IP_LEN];
     Port udpCtrlPort;
     Port tcpCtrlPort;
} SockCtrlHello;

typedef struct SockCtrlFarewell {
     char ipAddr[SOCK_MAX_IP_LEN];
} SockCtrlFarewell;

typedef struct SockCtrlSetRate {
     int rate;
} SockCtrlSetRate;

typedef struct SockCtrlSetRemote {
     Port port;
} SockCtrlSetRemote;

typedef struct SockCtrlAck {
	int numBytes;
} SockCtrlAck;

typedef struct SockCtrlNewCtrlChannel {
     int protocol;
     char ipAddr[SOCK_MAX_IP_LEN];
     int port;
} SockCtrlNewCtrlChannel;

typedef struct SockCtrlPkt {
     int reqType;
     struct sockaddr_in srcAddr;
     union {
	  SockCtrlSetRate             	setRate;
	  SockCtrlSetRemote           	setRemote;
	  SockCtrlNewCtrlChannel  	newCtrlChannel;
	  SockCtrlFarewell		farewell;
	  SockCtrlHello			hello;
	  SockCtrlAck			ack;
     } data;
} SockCtrlPkt;


typedef int PROTOCOLS;


typedef struct RemoteHost {
  int active;
  char *remoteName;
  char remoteIPAddr[SOCK_MAX_IP_LEN];
  Address udpRSin;
  Port udpCtrlPort;
  Port tcpCtrlPort;
  struct Socket *tcpCtrl;
  struct RemoteHost *next;
} RemoteHost;

#endif
