/*
 * Copyright (c) 1996-1998 Regents of the University of California.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the SPAND Research
 *      Group at the University of California Berkeley.
 * 4. Neither the name of the University nor of the Research Group may be
 *    used to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
/* $Header: /home/stemm/cvs/Socket++/Tcp.cc,v 1.6 1998/05/28 20:15:30 stemm Exp $ */
#include <stdio.h>
#include <string.h>
#include <strings.h>
#include <stdlib.h>
#include <signal.h>
#include <fcntl.h>
#include <errno.h>
#include <assert.h>
#include <sys/file.h>
#include <sys/stat.h>
#include <math.h>
#include <unistd.h>
#include "porting.h"
#include "dbgprintlib.h"
#include "Socket.h"
#include "Tcp.h"

TcpSocket::TcpSocket(int isCtrl = SOCK_NOT_CTRL) : Socket(isCtrl)
{
    if (InitSocket() != SOCK_SUCCESS) {
	state_ = SOCK_STATE_CLOSED;
	return;
    }
}
    
TcpSocket::TcpSocket() : Socket(SOCK_NOT_CTRL)
{
    if (InitSocket() != SOCK_SUCCESS) {
	state_ = SOCK_STATE_CLOSED;
	return;
    }
}

Sock_Status
TcpSocket::InitSocket()
{
    struct protoent *proto;
#ifdef THREADED
    struct protoent protoTemp;
    char tempstr[256];
#endif
    char *protoStr;
    int i=1, j=sizeof(i);

    protocol_ = SOCK_TCP;
    type_ = SOCK_STREAM;
    protoStr="tcp";
#ifdef THREADED
    proto = &protoTemp;
    proto = getprotobyname_r(protoStr, proto, tempstr, 256);
#else
    proto = getprotobyname(protoStr);
#endif

    /* get the socket */
    if ((rfd_ = sfd_ = socket(AF_INET, type_, proto->p_proto)) == -1) {
	debug_printf(110, "TcpSock_socket: ERR opening socket");
	endprotoent();
	return SOCK_FAILURE;
    }
    endprotoent();
    SetNoDelay(1);
    if (setsockopt(sfd_, SOL_SOCKET, SO_REUSEADDR, (char *)&i,j) < 0) {
	debug_printf(110, "TcpSock_Socket: ERR reuseaddr\n");
	return SOCK_FAILURE;
    }
    SetBufferSize(SOCK_DEFAULT_BUF_SIZE);

#ifdef HAVE_LIBSOCKS
    useSocks_=0;
#endif
    return SOCK_SUCCESS;
}

void
TcpSocket::SetMSS(int value)
{
    int i, j = sizeof(value);
    struct protoent *proto = getprotobyname("tcp");

    if (value < 500)
	debug_printf(109,"TcpSock_Setopt: low mss\n");
    if (setsockopt(sfd_,proto->p_proto, 
		   TCP_MAXSEG,(char *)&value,j) < 0)
	debug_printf(110, "TcpSock_SetOpt: set mss");
    if (getsockopt(sfd_,proto->p_proto,
#if (defined LINUX)		   
		   TCP_MAXSEG,(char *)&i,(unsigned int *) &j) < 0)
#else
		   TCP_MAXSEG,(char *)&i,&j) < 0)
#endif      
	debug_printf(110, "TcpSock_SetOpt: get mss");
    debug_printf(109,"TcpSock_Setopt: set mss to %d\n", i);
}

int 
TcpSocket::Connect(char *hostname, Port port)
{

    Socket::Connect(hostname, port);
    
    if (connect_(sfd_, (struct sockaddr *)&rSin_, sizeof(rSin_)) == -1){
	debug_printf(110, "TcpSock_connect: ERR connecting");
	return(SOCK_FAILURE);
    }
    if (compatible_ == SOCK_NOT_COMPATIBLE && isCtrl_ != SOCK_CTRL) {
	int status;

	if ((status = SendHello()) != SOCK_SUCCESS)
	    return(status);
	if ((status = RecvHello()) != SOCK_SUCCESS)
	    return(status);
    }
    numActiveRHost_=1;

    SetSocketInfo();
    state_ = SOCK_STATE_CONNECTED;
    return(SOCK_SUCCESS);

}

int 
TcpSocket::Listen(Port port)
{
    int temp;
    
    temp = Socket::Listen(port);
    if (temp != SOCK_SUCCESS)
	return(temp);
    if ((temp = listen_(rfd_, 5)) == -1) {
	debug_printf(110, "TcpSock_listen: ERR listening\n");
	return(SOCK_FAILURE);
    }
    state_ = SOCK_STATE_LISTENING;
    return(SOCK_SUCCESS);
}

TcpSocket *
TcpSocket::Accept()
{
    int temp;
    TcpSocket *cdPtr;

    cdPtr = new TcpSocket(isCtrl_);

    /* Close the fd associated with the socket */
    /* (because the accept creates a new one) */
    close(cdPtr->GetFD());
    
    *cdPtr = *this;
    temp = cdPtr->AcceptOW();
    if (temp != SOCK_SUCCESS) {
	delete cdPtr;
	cdPtr = NULL;
    }
    return(cdPtr);
}

Sock_Status
TcpSocket::AcceptOW()
{
    int ts;
    int addrLen;

    debug_printf(112, "TcpSock_Accept\n");
    Socket::AcceptOW();

    addrLen = sizeof(rSin_);
    if ((ts = accept_(rfd_, (struct sockaddr *)&(rSin_), &addrLen))==-1) {
	debug_printf(110, "TcpSock_accept: ERR accepting connection");
	return(SOCK_FAILURE);
    }
    rfd_ = ts;
    sfd_ = ts;
    if (compatible_ == SOCK_NOT_COMPATIBLE) {
	int status;
	/* expect a "connect me" packet */
	if ((status = RecvHello()) != SOCK_SUCCESS)
	    return(status);
	/* send and "accepted" packet */
	if ((status = SendHello()) != SOCK_SUCCESS)
	    return(status);
    }
    SetRemoteByAddr(&rSin_);
    numActiveRHost_=1;

    debug_printf(109,"connected from inet port %d to %d tcp, %d udp\n",
	    localPort_, rHost_->tcpCtrlPort, rHost_->udpCtrlPort );
    side_ = SOCK_SIDE_SERVER;
    state_ = SOCK_STATE_ACCEPTED;
    return(SOCK_SUCCESS);
}


int 
TcpSocket::Recv(char *buffer, int length)
{
    int status;

    debug_printf(112, "TcpSock_Recv %d\n", length);
    if ((status = WaitForPktRecv()) != SOCK_SUCCESS)
	return status;
    status = RecvStream(buffer, length);
    return(status);
}

int 
TcpSocket::RecvFull(char *buffer, int length)
{
    int status;

    debug_printf(112, "TcpSock_Recv %d\n", length);
    if ((status = WaitForPktRecv()) != SOCK_SUCCESS)
	return status;
    status = RecvStreamFull(buffer, length);
    return(status);
}

int 
TcpSocket::Send(char *buffer, int length)
{
    int status;

    debug_printf(112, "TcpSock_Send %d\n", length);
    WaitForPktSend(length);
    status = SendStream(buffer, length);
    if (status < 0) 
	lastPktSize_ = 0;
    return (status);
}



