/*
 * Copyright (c) 1996-1998 Regents of the University of California.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the SPAND Research
 *      Group at the University of California Berkeley.
 * 4. Neither the name of the University nor of the Research Group may be
 *    used to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
/* $Header: /home/stemm/cvs/Socket++/Tcp.h,v 1.5 1998/06/08 15:14:55 ss Exp $ */
#ifndef SS_SOCKET_TCP
#define SS_SOCKET_TCP
#include "Socket.h"

#ifdef HAVE_LIBSOCKS5
#	define SOCKS 5
#	include <socks.h>
#else
#	ifdef HAVE_LIBSOCKS
extern "C" {
int SOCKSinit(char *);
int Raccept(int, struct sockaddr *, int *);
int Rbind(int, struct sockaddr *, int, unsigned long);
int Rconnect(int, struct sockaddr *, int);
int Rlisten(int, int);
int Rgetsockname(int, struct sockaddr *, int *);
int Rselect(int, fd_set *, fd_set *, fd_set *, struct timeval *);
}
#	endif
#endif

/** Class that supports basic TCP communications between a pair of hosts */
class TcpSocket : public Socket {
public:
    TcpSocket();
    TcpSocket(int isCtrl);
    /** connect socket to remote socket */
    Sock_Status Connect(char *hostname, Port port);
    /** listen for connections on port */
    Sock_Status Listen(Port port);
    /** recv will recv at most length bytes */
    Sock_Status Recv(char *buffer, int length);
    /** recvfull will recv exactly length bytes */
    Sock_Status RecvFull(char *buffer, int length);
    /** send length bytes from buffer */
    Sock_Status Send(char *buffer, int length);
    /** accept a connection on a port 
      return val: socket connected to hostname */
    TcpSocket* Accept();
    /** accept a connection on a port -- socket cannot be accepted again*/
    Sock_Status AcceptOW();
    /** set the max seg size for the connection */
    void SetMSS(int value);
    /** set the tcp no delay option */
    inline int SetNoDelay(int value) 
	{return(SetSockOpt(TCP_NODELAY,value,"tcp"));}
    int operator==(TcpSocket b);

#ifdef TCPSTATS
    /** enable the the usage of TCP stats in the kernel */
    inline int SetTCPStats(int optflag) 
	{return(SetSockOpt(TCP_ENABLE_STATS, optflag, "tcp"));}
#endif
#ifdef HAVE_LIBSOCKS
    /** set the usage of the SOCKS */
    inline Sock_Status SetUseSocks(int value) {
	static int socksInit = 0;
	useSocks_ = value;
	if (value & !socksInit) {
	    SOCKSinit("Socket++ Lib");
	    socksInit = 1; }
	return SOCK_SUCCESS;};
    /** get the usage setting for SOCKS */
    inline int GetUseSocks() {return useSocks_ ;};
    /** set the remote machine that will connect to a socks accept */
    inline Sock_Status SetSocksRhost(char *hostname) 
	{ Address addr;
	if (Sock_NameToSin(hostname, 1000, &addr) < 0)
	    return SOCK_FAILURE;
	socksRhost_ = addr.sin_addr.s_addr;
	return SOCK_SUCCESS;
	};
#else 
    /** set the usage of the SOCKS */
    inline Sock_Status SetUseSocks(int value) {return SOCK_FAILURE;};
    /** get the usage setting for SOCKS */
    inline int GetUseSocks() {return SOCK_FAILURE;};
    /** set the remote machine that will connect to a socks accept */
    inline Sock_Status SetSocksRhost(char *hostname) 
	{return SOCK_FAILURE;};
#endif

protected:
    Sock_Status InitSocket();
#ifdef HAVE_LIBSOCKS
    int useSocks_;
    unsigned long socksRhost_;
    virtual inline int accept_(int fd, struct sockaddr *sin, int *len) 
	{if (useSocks_) return(Raccept(fd, sin, len));
#ifdef AIX4
	return(accept(fd, sin, (long unsigned int *)len));};
#else
	return(accept(fd, sin, len));};
#endif
    virtual inline int connect_(int fd, struct sockaddr *sin, int len)
	{if (useSocks_) return(Rconnect(fd, sin, len));
	return(connect(fd, sin, len));};
    virtual inline int bind_(int fd, struct sockaddr *sin, int len)
	{if (useSocks_) return(Rbind(fd, sin, len, socksRhost_));
	return (bind(fd, sin, len));};
    virtual inline int listen_(int fd, int count)
	{if (useSocks_) return(Rlisten(fd, count));
	return(listen(fd, count));};
    virtual inline int getsockname_(int fd, struct sockaddr *sin, int *len)
	{if (useSocks_) return(Rgetsockname(fd, sin, len));
#ifdef AIX4
	return(getsockname(fd, sin, (long unsigned int *)len));};
#else
	return(getsockname(fd, sin, len));};
#endif
    virtual inline int select_(int width, fd_set *rfds, fd_set *wfds, 
				   fd_set *efds, struct timeval *tout)
	{if (useSocks_) return(Rselect(width, rfds, wfds, efds, tout));
	return(select(width, rfds, wfds, efds, tout));};
#endif

};

#endif
