/*
 * Copyright (c) 1996-1998 Regents of the University of California.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the SPAND Research
 *      Group at the University of California Berkeley.
 * 4. Neither the name of the University nor of the Research Group may be
 *    used to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
/* $Header: /home/stemm/cvs/Socket++/Udp.cc,v 1.5 1998/05/28 20:15:30 stemm Exp $ */
#include <stdio.h>
#include <string.h>
#include <strings.h>
#include <stdlib.h>
#include <signal.h>
#include <fcntl.h>
#include <errno.h>
#include <assert.h>
#include <sys/file.h>
#include <sys/stat.h>
#include <math.h>
#include "porting.h"
#include "dbgprintlib.h"
#include "Socket.h"
#include "Udp.h"

UdpSocket::UdpSocket(int isCtrl = SOCK_NOT_CTRL) : Socket(isCtrl)
{
    if (InitSocket() != SOCK_SUCCESS) {
	state_ = SOCK_STATE_CLOSED;
	return;
    }
}
    
UdpSocket::UdpSocket() : Socket(SOCK_NOT_CTRL)
{
    if (InitSocket() != SOCK_SUCCESS) {
	state_ = SOCK_STATE_CLOSED;
	return;
    }
}

Sock_Status
UdpSocket::InitSocket()
{
    struct protoent *proto;
    int i=1, j=sizeof(i);
    char *protoStr;
#ifdef THREADED
    char tempstr[256];
    struct protoent protoTemp;
#endif

    protocol_ = SOCK_UDP;
    type_ = SOCK_DGRAM;

    protoStr="udp";
#ifdef THREADED
    proto = &protoTemp;
    proto = getprotobyname_r(protoStr, proto, tempstr, 256);
#else
    proto = getprotobyname(protoStr);
#endif

    /* get the socket */
    if ((rfd_ = sfd_ = socket(AF_INET, type_, proto->p_proto)) == -1) {
	debug_printf(110, "UdpSock_socket: ERR opening socket");
	endprotoent();
	return SOCK_FAILURE;
    }
    endprotoent();
    if (setsockopt(sfd_, SOL_SOCKET, SO_REUSEADDR, (char *)&i,j) < 0)
	debug_printf(110, "UdpSock_Socket: ERR reuseaddr\n");
    if (setsockopt(sfd_, SOL_SOCKET, SO_BROADCAST, (char *)&i,j) < 0)
	debug_printf(110, "UdpSock_Socket: ERR so_broadcast\n");
    SetBufferSize(SOCK_DEFAULT_BUF_SIZE);

    mtu_ = 1500;
    return SOCK_SUCCESS;
}

int 
UdpSocket::Connect(char *hostname, Port port)
{
    Socket::Connect(hostname, port);
    
    if (state_ == SOCK_STATE_ALLOCATED) {
	lSin_.sin_family = AF_INET;
	lSin_.sin_port = htons(0);
	lSin_.sin_addr.s_addr = htonl(INADDR_ANY);
	if ((bind_(sfd_,
		  (struct sockaddr *)&lSin_,
		  sizeof(lSin_))) == -1) {
	    debug_printf(110, "UdpSock_connect: ERR binding socket");
	    return(SOCK_FAILURE);
	}
    }

    if (compatible_ == SOCK_NOT_COMPATIBLE && isCtrl_ != SOCK_CTRL) {
	int status;

	if ((status = SendHello()) != SOCK_SUCCESS)
	    return(status);
	if ((status = RecvHello()) != SOCK_SUCCESS)
	    return(status);
    }
    numActiveRHost_=1;

    SetSocketInfo();
    state_ = SOCK_STATE_CONNECTED;
    return(SOCK_SUCCESS);

}

int 
UdpSocket::Listen(Port port)
{
    int temp;
    
    temp = Socket::Listen(port);
    if (temp != SOCK_SUCCESS)
	return(temp);
    state_ = SOCK_STATE_LISTENING;
    return(SOCK_SUCCESS);
}

UdpSocket *
UdpSocket::Accept()
{
    int temp;
    UdpSocket *cdPtr;

    cdPtr = new UdpSocket(isCtrl_);
    
    *cdPtr = *this;
    temp = cdPtr->AcceptOW();
    if (temp != SOCK_SUCCESS) {
	delete cdPtr;
	cdPtr = NULL;
    } else {
	rfd_ = -1;
	sfd_ = -1;
    }
    return(cdPtr);
}

Sock_Status
UdpSocket::AcceptOW()
{

    debug_printf(112, "UdpSock_Accept\n");
    Socket::AcceptOW();

    if (compatible_ == SOCK_NOT_COMPATIBLE) {
	int status;
	/* expect a "connect me" packet */
	if ((status = RecvHello()) != SOCK_SUCCESS)
	    return(status);
	SetRemoteByAddr(&(lastRSin_));
	/* send and "accepted" packet */
	if ((status = SendHello()) != SOCK_SUCCESS)
	    return(status);
    }
    numActiveRHost_=1;

    debug_printf(109,"connected from inet port %d to %d tcp, %d udp\n",
	    localPort_, rHost_->tcpCtrlPort, rHost_->udpCtrlPort );
    side_ = SOCK_SIDE_SERVER;
    state_ = SOCK_STATE_ACCEPTED;
    return(SOCK_SUCCESS);
}

int 
UdpSocket::Recv(char *buffer, int length)
{
    int status;

    debug_printf(112, "UdpSock_Recv %d\n", length);
    if ((status = WaitForPktRecv()) != SOCK_SUCCESS)
	return status;
    status = RecvDgram(buffer, length);
    return status;
}

int
UdpSocket::RecvFull(char *buffer, int length)
{
    int status;

    debug_printf(112, "UdpSock_Recv %d\n", length);
    if ((status = WaitForPktRecv()) != SOCK_SUCCESS)
	return status;
    status = RecvDgramFull(buffer, length);
    return status;
}

int 
UdpSocket::Send(char *buffer, int length)
{
    int status;

    debug_printf(112, "UdpSock_Send %d\n", length);
    WaitForPktSend(length);
    status = SendDgram(buffer, length);
    if (status < 0) 
	lastPktSize_ = 0;
    return (status);
}

