/*
 * Copyright (c) 1996-1998 Regents of the University of California.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the SPAND Research
 *      Group at the University of California Berkeley.
 * 4. Neither the name of the University nor of the Research Group may be
 *    used to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
/* $Header: /home/stemm/cvs/Socket++/Utility.cc,v 1.4 1998/05/28 20:15:31 stemm Exp $ */
#include <stdio.h>
#include <string.h>
#include <strings.h>
#include <stdlib.h>
#include <signal.h>
#include <fcntl.h>
#include <errno.h>
#include <assert.h>
#include <sys/file.h>
#include <sys/stat.h>
#include <math.h>
#include "porting.h"
#include "dbgprintlib.h"
#include "Socket.h"
#include "Utility.h"

void *
Sock_Malloc(int size)
{
     void *mem;
     if ((mem = malloc((size_t) size)) == NULL)
	  perror("Sock_Malloc: malloc failed");
     return mem;
}

void 
Sock_SigPipeHandle()
{
     signal(SIGPIPE, SIG_IGN);
}


int 
Sock_DelayPacket(timeval *lastTimePtr, int lastSize, int rate, int size)
{
     struct timeval curTime, nextTime, time1, time2;
     int usecWait=0;
     static minUsleep = SOCK_MIN_USLEEP;
     int temp;
     double tempd;
     
     tempd = (double)lastSize * 1000000/rate;
     temp = (int) (lastTimePtr->tv_usec + tempd);
     nextTime.tv_sec = lastTimePtr->tv_sec + (int)(temp/1000000);
     nextTime.tv_usec = temp % 1000000;
     gettimeofday(&curTime, NULL);
     usecWait = ((nextTime.tv_sec - curTime.tv_sec) * 1000000) +
	       nextTime.tv_usec- curTime.tv_usec;
     debug_printf(110,"nxt us %d lst us %d wait %d lps %d temp %d\n",
	     nextTime.tv_usec, lastTimePtr->tv_usec, 
	     usecWait, lastSize, temp);
     if (usecWait < 0) {
	  lastTimePtr->tv_sec = curTime.tv_sec;
	  lastTimePtr->tv_usec = curTime.tv_usec;
	  return(size);
     }
     if (usecWait > 0 && usecWait > minUsleep) {
	  gettimeofday(&time1, NULL);
	  microsleep(usecWait);
	  gettimeofday(&time2, NULL);
	  temp = (time2.tv_usec - time1.tv_usec) +
	       (time2.tv_sec - time1.tv_sec) * 1000000;
	  debug_printf(110, "usecWait %d realWait %d minUsleep %d\n", 
		  usecWait, temp, minUsleep);
	  if (temp > 1.25 * usecWait && temp > minUsleep) {
	       minUsleep = (int) (1.25 * temp);
	  }
  	  lastTimePtr->tv_sec = nextTime.tv_sec;
	  lastTimePtr->tv_usec = nextTime.tv_usec;
	  return(size);
     }
     return(lastSize + size);
}


char *
Sock_BroadcastAddr(char *normAddr)
{
     char *newAddr;
     int i,dotCount=0, length;

     length = strlen(normAddr) + 3;
     newAddr = (char *)calloc(length, sizeof(char));
     for (i=0; i< length; i++) {
          if (((normAddr[i] < '0') || (normAddr[i] > '9')) && (normAddr[i] != '.')) {
               free(newAddr);
               return(NULL);
          }
          newAddr[i]=normAddr[i];
          if (normAddr[i] == '.') {
               dotCount++;
               if (normAddr[i+1] == '.') {
                    free(newAddr);
                    return(NULL);
               }
          }
          if (dotCount == 3)
               break;
     }
     newAddr[i+1] = '0';
     newAddr[i+2] = '0';
     newAddr[i+3] = '0';
     debug_printf(109,"set broadcast address to %s\n", newAddr);
     return(newAddr);
}

int 
Sock_IsIPAddr(char *id)
{
#ifndef INADDR_NONE
#define INADDR_NONE -1
#endif
     return (inet_addr(id) != INADDR_NONE) ;
}

int
Sock_NameToSin(char *hostname, Port port, Address *sinPtr) 
{
     u_int hostAddr;
     struct hostent *hp;
     char tempstr[1024];
#ifdef THREADED
     int temp;
     struct hostent hpt;
#endif

     if ((hostAddr = inet_addr(hostname)) != INADDR_NONE) {
	  debug_printf(109, "getting host info from addr\n");
	  memcpy(&(sinPtr->sin_addr), &hostAddr, sizeof(hostAddr));
     }
     else {
	  debug_printf(109, "getting host info from name\n");
#ifdef THREADED
	  hp = &hpt;
	  hp = gethostbyname_r(hostname,
			  hp, tempstr, 1024, &temp);
#else
	  hp = gethostbyname(hostname);
#endif
	  if (hp == NULL) {
	       sprintf(tempstr, "error setting remote host to %s", hostname);
	       debug_printf(110, tempstr);
	       endhostent();
	       return SOCK_FAILURE;
	  }
	  memcpy(&(sinPtr->sin_addr.s_addr), hp->h_addr, hp->h_length);
	  endhostent();
     }

     sinPtr->sin_family = AF_INET;
     sinPtr->sin_port = htons(port);
     return SOCK_SUCCESS;
}

int 
Sock_SinToName(char *hostnamePtr, int length, Port *port, Address *sinPtr) 
{
#ifdef THREADED
     char tempstr[1024];
     struct hostent hpt;
     int temp;
#endif
     struct hostent *hp;

     *port = ntohs(sinPtr->sin_port);
     if (IN_CLASSD(ntohl(sinPtr->sin_addr.s_addr))) {
	  hp = NULL;
     } else {
#ifdef THREADED
	  hp = &hpt;
	  hp = gethostbyaddr_r((char *)&(sinPtr->sin_addr), 
			       sizeof(struct in_addr), AF_INET,
			       hp, tempstr, 1024, &temp);
#else
	  hp = gethostbyaddr((char *)&(sinPtr->sin_addr), 
			     sizeof(struct in_addr), AF_INET);
#endif
     }
     if ( hp == NULL) {
	  endhostent();
	  debug_printf(109,"setting remote to %s\n", inet_ntoa(sinPtr->sin_addr));
	  strncpy(hostnamePtr, inet_ntoa(sinPtr->sin_addr), (size_t) length);
	  debug_printf(110, "couldn't set remote name by addr");
	  return SOCK_SUCCESS;
     }
     debug_printf(109, "setting remote to %s\n", hp->h_name);
     strncpy(hostnamePtr, hp->h_name, (size_t) length);
     endhostent();
     return SOCK_SUCCESS;
}

timev *
Sock_UtimeToTval(int microsecs)
{
     timev *tv;

     tv = (timev *) Sock_Malloc(sizeof (timev));
     tv->tv_usec = microsecs%1000000;
     tv->tv_sec = microsecs/1000000;
     return tv;
}

int 
Sock_TvalToUtime(timev *tv)
{
     int usec;
     
     if (tv == NULL)
	 return 0;
     usec = tv->tv_usec +  tv->tv_sec * 1000000;
     return usec;
}

int 
Sock_TimeDiff(timev t1, timev t2)
{
     return ((t1.tv_sec - t2.tv_sec)*1000000 + (t1.tv_usec - t2.tv_usec));
}
