/*
 * Copyright (c) 1996-1998 Regents of the University of California.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the SPAND Research
 *      Group at the University of California Berkeley.
 * 4. Neither the name of the University nor of the Research Group may be
 *    used to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
/* $Header: /home/stemm/cvs/SPAND/src/client/client.cc,v 1.11 1998/06/10 23:18:46 stemm Exp $ */
#include <stdio.h>
#include <string.h>
#include <sys/time.h>
#include <math.h>
#include <strings.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <unistd.h>
#include <errno.h>

#include "porting.h"
#include <Classes.h>
#include <Socket.h>
extern "C" {
#include "dbgprintlib.h"
#include "option.h"  
}
#include <Spand.h>

char *spandSvr="127.0.0.1";
int perf=0;
int raw=0;
int summ=0;
char *qaddrStr="127.0.0.1";

Option optionArray[] = {
  {OPT_TRUE, "perf", (char *) &perf, "Get Performance For Host"},
  {OPT_TRUE, "raw", (char *) &raw, "Get Raw Reports For Host"},
  {OPT_TRUE, "summ", (char *) &summ, "Get Summary of Reports For Host"},
  {OPT_STRING, "perfSvr", (char *) &spandSvr, "Name/Address of Spand Perf Server"},
  {OPT_STRING, "addr", (char *) &qaddrStr, "Address to ask about"}
};

#define USAGESTR "usage: client [-addr <addr>] [-perf| -raw | -summ] [-perfSvr <name/addr>]\n"

/** @name client program
    This is a simple SPAND client that queries the Performance Server
    for information on a particular address. This client assumes that the Performance Server is localhost.
    
    The usage is: {\tt client [-addr <addr>] [-perf | -raw |-summ] [-perfSvr <name/addr>]}
    
    {\tt [-addr <addr>]} is the IP address (not name) for which to query the performance.

    {\tt -perf} means to get a Performance Response for that address.
    
    {\tt -raw} means to get all Performance Reports for that address.
    
    {\tt -summ} means to return the number of Performance Reports collected for that address.

    {\tt [-perfSvr <name/address>]} is the IP address or name of the SPAND Performance Server.

    * @author SPAND Group, UC Berkeley.
 * @version $Revision: 1.11 $.
 */
    
main(int argc, char *argv[])
{
  Addr *qaddr;
  Msg *requestMsg;
  Msg *replyMsg;
  PerfRequestPayload *p;
  Requestor *requestor;
  
  argc=Opt_Parse(argc, argv, optionArray, Opt_Number(optionArray), 0);

  qaddr = new Addr(qaddrStr);
  p = new PerfRequestPayload(*qaddr);
  requestor=new Requestor(spandSvr);
  
  if(perf && !raw && !summ){
    requestMsg = new Msg("spand.GetPerf", *p, 1);
  } else if (!perf && raw && !summ){
    requestMsg = new Msg("spand.GetRaw", *p, 1);
  } else if (!perf && !raw && summ){
    requestMsg = new Msg("spand.GetSumm", *p, 1);
  } else {
    fprintf(stderr, USAGESTR);
    exit(-1);
  }

  replyMsg = requestor->Request(requestMsg, 1);
  
  printf("%s\n", replyMsg->Print());
}
