/*
 * Copyright (c) 1996-1998 Regents of the University of California.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the SPAND Research
 *      Group at the University of California Berkeley.
 * 4. Neither the name of the University nor of the Research Group may be
 *    used to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
/* $Header: /home/stemm/cvs/dbgprint/dbgprint.c,v 1.4 1998/06/03 20:30:55 stemm Exp $ */
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <varargs.h>

#define DP_PRINT 0
#define DP_SET_LEVEL 1
#define DP_SET_FD 2
#define DP_FLUSH_FD 3
#define DP_SET_PE_LEVEL 4
#define DP_PERROR 5

int _debug_printf(command, filename, level, format, ap)
int command;
char *filename;
int level;
const char *format;
va_list ap;
{
  static int currentDebugLevel = 0;
  static int currentPerrorLevel = 0;
  static FILE *f = stderr;

  switch (command){
  case DP_PRINT:
      if (currentDebugLevel >= level)
	  return(vfprintf(f, format, ap));
      else return(0);
  case DP_SET_LEVEL:
      currentDebugLevel = level;
      return 0;
  case DP_SET_PE_LEVEL:
      currentPerrorLevel = level;
      return 0;
  case DP_PERROR:{
      char *error = strerror(errno);
      int i;

      if (currentPerrorLevel >= level) {
	  fprintf(f, "DPERROR - ");
	  i = vfprintf(f, format, ap);
	  fprintf(f, ": %s\n", error);
	  return i;
      }
      return(0);
  }
  case DP_SET_FD:
      if (f != stderr)
	  fclose(f);
      f = fopen(filename, "w");
      if (f == 0) {
	  perror(filename);
	  f = stderr;
	  return -1;
      }
      return 0;
  case DP_FLUSH_FD:
      fflush(f);
      return 0;
  default:
      return -1;
  }
}

int 
debug_printf(level, format, va_alist)
int level;
const char *format;
va_dcl
{
  int n = 0;
  va_list ap;
  
  va_start(ap); 
  n = _debug_printf(DP_PRINT, NULL, level, format, ap);
  va_end(ap); 
  return n;
}

int
debug_perror(level, format, va_alist)
int level;
const char *format;
va_dcl
{
  int n = 0;
  va_list ap;

  va_start(ap); 
  n = _debug_printf(DP_PERROR, NULL, level, format, ap);
  va_end(ap); 
  return n;
}

void
SetDebugLevel(int level)
{
    _debug_printf(DP_SET_LEVEL, NULL, level, "debug level set to %d\n", level);
}

int
SetDebugFile(char *filename)
{
    return(_debug_printf(DP_SET_FD, filename, 
		   "setting file for debugging to %s", filename));
}

int 
SetDperrorLevel(int level)
{
    _debug_printf(DP_SET_PE_LEVEL, NULL, level, "debug level set to %d\n", level);
}

int
FlushDebugFile()
{
    return(_debug_printf(DP_FLUSH_FD, NULL,"", NULL));
}











