/*
 * Copyright (c) 1996-1998 Regents of the University of California.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the SPAND Research
 *      Group at the University of California Berkeley.
 * 4. Neither the name of the University nor of the Research Group may be
 *    used to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
/* $Header: /home/stemm/cvs/SPAND/src/libspand/Spand.cc,v 1.20 1998/06/27 01:42:08 stemm Exp $ */
#include <Spand.h>



Addr::Init(struct in_addr addr0)
{
	addr.s_addr=addr0.s_addr;
}

Addr::Addr()
{
	struct in_addr temp;
	temp.s_addr=0;
	Init(temp);
}

Addr::Addr(struct in_addr addr0)
{
	Init(addr0);
}

Addr::Addr(char *addr)
{
	struct in_addr temp;
	temp.s_addr=ntohl(inet_addr(addr));
	Init(temp);
}

char *Addr::Print()
{
	struct in_addr temp;
	
	temp.s_addr=htonl(addr.s_addr);
	sprintf(buf, "%s", inet_ntoa(temp));
	
	return (char *) buf;
}

u_long Addr::Unmarshal(u_char *wireData)
{
	struct WireFormat *m = (struct WireFormat *) wireData;

	addr.s_addr=ntohl(m->addr.s_addr);
	
	return sizeof(struct WireFormat);
}

u_long Addr::Marshal(u_char *wireData)
{
	struct WireFormat *m = (struct WireFormat *) wireData;

	/* If passed a NULL pointer, return the size of the marshalled
           thing without doing anything */
	if (wireData==NULL){
		return sizeof(struct WireFormat);
	}
	
	m->addr.s_addr=htonl(addr.s_addr);
	return sizeof(struct WireFormat);
}

ReportPort::Init(u_short port0)
{
	port=port0;
}

ReportPort::ReportPort()
{
	Init(0);
}

ReportPort::ReportPort(u_short port0)
{
	Init(port0);
}

char *ReportPort::Print()
{
	sprintf(buf, "%d", port);
	return (char *) buf;
}

u_long ReportPort::Unmarshal(u_char *wireData)
{
	struct WireFormat *m = (struct WireFormat *) wireData;

	port=(u_short) ntohl(m->port);
	return sizeof(struct WireFormat);
}

u_long ReportPort::Marshal(u_char *wireData)
{
	struct WireFormat *m = (struct WireFormat *) wireData;

	/* If passed a NULL pointer, return the size of the marshalled
           thing without doing anything */
	if (wireData==NULL){
		return sizeof(struct WireFormat);
	}
	
	m->port=htonl((u_long) port);
	return sizeof(struct WireFormat);
}


char * AddrPort::Print()
{
	sprintf(buf, "%s.%s", addr.Print(), port.Print());
	return (char *) buf;
}

u_long AddrPort::Marshal(u_char *wireData)
{
	u_char *p=wireData;

	/* If passed a NULL pointer, return the size of the marshalled
           thing without doing anything */
	if (wireData==NULL){
		return addr.Marshal(NULL)+port.Marshal(NULL);
	}
	
	p+= addr.Marshal(p);
	p+= port.Marshal(p);
	return (u_long)p-(u_long) wireData;
}

u_long AddrPort::Unmarshal(u_char *wireData)
{
	u_char *p=wireData;

	p+= addr.Unmarshal(p);
	p+= port.Unmarshal(p);
	return (u_long)p-(u_long) wireData;
}

u_long FlowID::Marshal(u_char *wireData)
{
	u_char *p=wireData;

	/* If passed a NULL pointer, return the size of the marshalled
           thing without doing anything */
	if (wireData==NULL){
		return src.Marshal(NULL)+dst.Marshal(NULL);
	}
	
	p+= src.Marshal(p);
	p+= dst.Marshal(p);
	return (u_long)p-(u_long) wireData;
}

u_long FlowID::Unmarshal(u_char *wireData)
{
	u_char *p=wireData;

	p+= src.Unmarshal(p);
	p+= dst.Unmarshal(p);
	return (u_long)p-(u_long) wireData;
}

char *FlowID::Print()
{
	sprintf(buf, "%s -> %s", src.Print(), dst.Print());
	return (char *) buf;
}

SendableSpandObject::operator==(SendableSpandObject &b)
{
	u_char *m1, *m2;
	u_long aSize, bSize;
	int rc;
	
	m1 = new u_char (Marshal(NULL));
	m2 = new u_char (b.Marshal(NULL));
	
	aSize=Marshal(m1);
	bSize=b.Marshal(m2);
	
	if (aSize!=bSize){
		fprintf(stderr, "Two messages with different marshalled sizes?\n");
	}
	
	rc=memcmp(m1, m2, aSize);
	delete m1;
	delete m2;
	return(rc);
}

/* Initializes Fields */
ReportPayload::Init()
{
	/* Set all fields to zero */
	mostSigNTP=0;
	leastSigNTP=0;
	sampLen=0;
	sampDur=0;
	packetsReceived=0;
	packetsLost=0;
	packetSize=0;
	srtt=0;
	
}  

/* Default constructor */
ReportPayload::ReportPayload()
{
	Init();
}

/* Constructor from variables. All multi-byte fields are assumed to be in host order. */
ReportPayload::ReportPayload(FlowID id0)
			     
{
	
	/* First init all fields */
	Init();
	
	/* Initialize static fields */
	id=id0;
}

/* Prepares ReportPayload structure */
u_long ReportPayload::Unmarshal(u_char *wireData)
{
	struct WireFormat *m=(struct WireFormat *) wireData;

	id.Unmarshal((u_char *) &(m->id));
	mostSigNTP=ntohl(m->mostSigNTP);
	leastSigNTP=ntohl(m->leastSigNTP);
	sampLen=ntohl(m->sampLen);
	sampDur=ntohl(m->sampDur);
	packetsReceived=ntohl(m->packetsReceived);
	packetsLost=ntohl(m->packetsLost);
	packetSize=ntohl(m->packetSize);
	srtt=ntohl(m->srtt);
	
	return sizeof(struct WireFormat);
}

/* Prepares ReportPayload structure */
u_long ReportPayload::Marshal(u_char *wireData)
{
	
	struct WireFormat *m=(struct WireFormat  *) wireData;

	/* If passed a NULL pointer, return the size of the marshalled
           thing without doing anything */
	if (wireData==NULL){
		return sizeof(struct WireFormat);
	}
	
	/* Pointer to source addr is start of flow id */
	id.Marshal((u_char *) &(m->id));
	m->mostSigNTP=htonl(mostSigNTP);
	m->leastSigNTP=htonl(leastSigNTP);
	m->sampLen=htonl(sampLen);
	m->sampDur=htonl(sampDur);
	m->packetsReceived=htonl(packetsReceived);
	m->packetsLost=htonl(packetsLost);
	m->packetSize=htonl(packetSize);
	m->srtt=htonl(srtt);
	
	return sizeof(struct WireFormat);
}

char *ReportPayload::Print()
{
	sprintf(buf, "%s time %d.%d sampLen %d sampDur %d bandwidth %f packetsReceived %d packetsLost %d packetSize %d srtt %d",
		id.Print(),
		mostSigNTP,
		leastSigNTP,
		sampLen,
		sampDur,
		(float) (sampLen*8)/(float) sampDur,
		packetsReceived,
		packetsLost,
		packetSize,
		srtt);
	
	return (char *) buf;
}

PerfRequestPayload::Init()
{
}

/* Default constructor */
PerfRequestPayload::PerfRequestPayload()
{
	Init();
}

/* Constructor from variables. All multi-byte fields are assumed to be in host order. */
PerfRequestPayload::PerfRequestPayload(Addr addr0)
{
	
	/* First init all fields */
	Init();
	
	/* Initialize static fields */
	addr=addr0;
}

/* Prepares PerfRequestPayload structure */
u_long PerfRequestPayload::Unmarshal(u_char *wireData)
{
	struct WireFormat *m=(struct WireFormat *) wireData;

	addr.Unmarshal((u_char *) &(m->addr));
	
	return sizeof(struct WireFormat);
}

/* Prepares PerfRequestPayload structure */
u_long PerfRequestPayload::Marshal(u_char *wireData)
{
	
	struct WireFormat *m=(struct WireFormat  *) wireData;

	/* If passed a NULL pointer, return the size of the marshalled
	   thing without doing anything */
	if (wireData==NULL){
		return sizeof(struct WireFormat);
	}
	
	addr.Marshal((u_char *) &(m->addr));
	
	return sizeof(struct WireFormat);
}

char * PerfRequestPayload::Print()
{
	sprintf(buf, "addr=%s\n",
		addr.Print());
	
	return (char *) buf;
}

PerfResponsePayload::Init()
{
	availBandwidth=0;
	stdDevAvailBandwidth=0;
	packetLossProb=0;
	stdDevPacketLossProb=0;
	numReports=0;
	numBytes=0;
	avgSrtt=0;
	stdDevSrtt=0;
	
}

/* Default constructor */
PerfResponsePayload::PerfResponsePayload()
{
	Init();
}


int CompareFloats(const void *v1, const void *v2)
{
	const float *f1=(float *) v1, *f2=(float *) v2;

	if (*f1<*f2){
		return -1;
	} else if (*f1==*f2){
		return 0;
	} else {
		return 1;
	}
}

/* Prepares PerfResponsePayload structure */
u_long PerfResponsePayload::Unmarshal(u_char *wireData)
{
	struct WireFormat *m=(struct WireFormat *) wireData;

	addr.Unmarshal((u_char *) &(m->addr));
	availBandwidth=ntohl(m->availBandwidth);
	stdDevAvailBandwidth=ntohl(m->stdDevAvailBandwidth);
	packetLossProb=ntohl(m->packetLossProb);
	stdDevPacketLossProb=ntohl(m->stdDevPacketLossProb);
	numReports=ntohl(m->numReports);
	numBytes=ntohl(m->numBytes);
	avgSrtt=ntohl(m->avgSrtt);
	stdDevSrtt=ntohl(m->stdDevSrtt);

	
	return sizeof(struct WireFormat);
}

/* Prepares PerfResponse structure */
u_long PerfResponsePayload::Marshal(u_char *wireData)
{
	
	struct WireFormat *m=(struct WireFormat  *) wireData;

	/* If passed a NULL pointer, return the size of the marshalled
	   thing without doing anything */
	if (wireData==NULL){
		return sizeof(struct WireFormat);
	}
	
	addr.Marshal((u_char *) &(m->addr));
	m->availBandwidth=htonl(availBandwidth);
	m->stdDevAvailBandwidth=htonl(stdDevAvailBandwidth);
	m->packetLossProb=htonl(packetLossProb);
	m->stdDevPacketLossProb=htonl(stdDevPacketLossProb);    
	m->numReports=htonl(numReports);
	m->numBytes=htonl(numBytes);
	m->avgSrtt=htonl(avgSrtt);
	m->stdDevSrtt=htonl(stdDevSrtt);
	
	return sizeof(struct WireFormat);
}

char * PerfResponsePayload::Print()
{
	sprintf(buf, "addr=%s bw=%u stddevBW=%u loss=%u stddev=%u numReports=%u numBytes=%u avgSrtt=%u stdDevSrtt=%u",
		addr.Print(),
		availBandwidth,
		stdDevAvailBandwidth,
		packetLossProb,
		stdDevPacketLossProb,
		numReports,
		numBytes,
		avgSrtt,
		stdDevSrtt);
	
	return (char *) buf;
}

SummResponsePayload::Init()
{
	numReports=0;
	
}

/* Default constructor */
SummResponsePayload::SummResponsePayload()
{
	Init();
}

/* Constructor from variables. All multi-byte fields are assumed to be in host order. */
SummResponsePayload::SummResponsePayload(Addr addr0,
					 u_long numReports0)
{
	/* First init all fields */
	Init();
	
	/* Initialize static fields */
	addr=addr0;
	
	numReports=numReports0;
}

/* Prepares SummResponsePayload structure */
u_long SummResponsePayload::Unmarshal(u_char *wireData)
{
	struct WireFormat *m=(struct WireFormat *) wireData;

	addr.Unmarshal((u_char *) &(m->addr));
	numReports=ntohl(m->numReports);
	
	return sizeof(struct WireFormat);
}

/* Prepares SummResponse structure */
u_long SummResponsePayload::Marshal(u_char *wireData)
{
	
	struct WireFormat *m=(struct WireFormat  *) wireData;

	/* If passed a NULL pointer, return the size of the marshalled
	   thing without doing anything */
	if (wireData==NULL){
		return sizeof(struct WireFormat);
	}
	
	addr.Marshal((u_char *) &(m->addr));
	m->numReports=htonl(numReports);
	
	return sizeof(struct WireFormat);
}

char * SummResponsePayload::Print()
{
	sprintf(buf, "addr=%s numReports=%u",
		addr.Print(),
		numReports);
	
	return (char *) buf;
}

ErrorPayload::ErrorPayload()
{
  errorMsg=new char(8);
  strcpy(errorMsg, "<empty>");
}

ErrorPayload::ErrorPayload(char *errorMsg0)
{
  errorMsg = new char(strlen(errorMsg0)+1);

  strcpy(errorMsg, errorMsg0);
}

ErrorPayload::~ErrorPayload()
{
  free(errorMsg);
}

u_long ErrorPayload::Marshal(u_char *wireData)
{
  int *temp;
  u_char *pos=wireData;
	
  /* If passed a NULL pointer, return the size of the marshalled
     thing without doing anything */
  if (wireData==NULL){
    /* 4 bytes for error string length */
    u_long unalign_size=4+strlen(errorMsg);
    while ((unalign_size % sizeof(u_long))!=0){
      unalign_size++;
    }
    return(unalign_size);
  }
  
  /* Write length of errorMsg string */
  temp=(int *) pos;
  *temp=htonl(strlen(errorMsg));
  pos+=sizeof(strlen(errorMsg));
  
  /* Write errorMsg string */
  memcpy(pos, errorMsg, strlen(errorMsg));
  pos+=strlen(errorMsg);

  /* To make sure things are word-aligned, write some additional bytes */
  while((((u_long) pos-(u_long) wireData) % sizeof(u_long))!=0){
    *pos=0;
    pos++;
  }

  return ((u_long) pos-(u_long) wireData);
}

u_long ErrorPayload::Unmarshal(u_char *wireData)
{
  u_char *pos=wireData;
  int *temp;
  int strLen;
	
  /* Read length of errorMsg string */
  temp=(int *) pos;
  strLen=ntohl(*temp);
  pos+=sizeof(strLen);

  errorMsg = malloc((strLen+1)*sizeof(char));
  
  /* Read errorMsg string */
  memcpy(errorMsg, pos, strLen);
  errorMsg[strLen]='\0';
  pos+=strLen;

  /* To make sure things are word-aligned, read some additional bytes */
  while((((u_long) pos-(u_long) wireData) % sizeof(u_long))!=0){
    *pos=0;
    pos++;
  }
  
  return ((u_long) pos-(u_long) wireData);
}

char *ErrorPayload::Print()
{
  sprintf(buf, "error=%s",errorMsg);
  return (char *) buf;
}

void Msg::ReadSock(Socket *readFrom, char *buf, int size)
{
	int temp;
	
	temp=readFrom->Recv((char *) buf, size);
	if (temp == SOCK_FAILURE) {
		debug_perror(1,"recvfull");
		empty=1;
	}
	if (temp == SOCK_FD_CLOSED) {
		debug_printf(0, "remote closed\n");
		empty=1;
	}
	
}

void Msg::ReadFullSock(Socket *readFrom, char *buf, int size)
{
	int temp;

	temp=readFrom->RecvFull((char *) buf, size);
	if (temp == SOCK_FAILURE) {
		debug_perror(1,"recvfull");
		empty=1;
	}
	if (temp == SOCK_FD_CLOSED) {
		debug_printf(0, "remote closed\n");
		empty=1;
	}
	
}

Msg::Msg(UdpSocket *readFrom)
{
	u_char buf[MAX_MSG_SIZE];
	u_char *p=buf;
	
	ReadSock(readFrom, (char *) buf, MAX_MSG_SIZE);

	Unmarshal(buf);
}

Msg::Msg(TcpSocket *readFrom)
{
	u_char buf[MAX_MSG_SIZE];
	int *temp;
	int handlerLen;
	u_long bytes_read=0;
	u_char tmpBuf[10];
	
	/* First read in the size of the handler string */
	ReadFullSock(readFrom, (char *) buf, 4);
	temp=(int *) buf;
	handlerLen=ntohl(*temp);
	bytes_read+=4;
	
	/* Now read in the handler string */
	ReadFullSock(readFrom, (char *) handler, handlerLen);
	handler[handlerLen]='\0';
	bytes_read+=handlerLen;

	/* To make sure things are word-aligned, add some additional bytes */
	while((bytes_read % sizeof(u_long))!=0){
	  ReadFullSock(readFrom, (char *) tmpBuf, 1);
	  bytes_read++;
	}
	
	/* Now read in the size of the data */
	ReadFullSock(readFrom, (char *) buf, 4);
	temp=(int *) buf;
	dataLen=ntohl(*temp);
	bytes_read+=4;
	
	/* Now read in the data */
	data=(unsigned char *) malloc(2*dataLen*sizeof(char));
	ReadFullSock(readFrom, (char *) data, dataLen);
	bytes_read+=dataLen;
}

Msg::~Msg()
{
  free(data);
}

Msg::Msg(Socket *readFrom)
{
  if (readFrom->GetType()==SOCK_UDP){
    Msg((UdpSocket *) readFrom);
  } else if (readFrom->GetType()==SOCK_TCP){
    Msg((TcpSocket *) readFrom);
  } else {
    fprintf(stderr, "Unknown socket type %d\n", readFrom->GetType());
  }
}

Msg::Msg(char *handler0, SendableSpandObject &o, int active0)
{
  strncpy(handler, handler0, MAX_MSG_SIZE-1);
  dataLen=o.Marshal(NULL);
  data=(unsigned char *) malloc(dataLen*sizeof(char));
  o.Marshal(data);

  active=active0;
}

char *Msg::Print()
{
  if (active) {
    sprintf(buf, "%s", handler);
  } else {
    SendableSpandObject *o=this.ToObject();
    sprintf(buf, "%s %s", handler, o->Print());
    delete o;
  }
  return(buf);
}

u_long Msg::Marshal(u_char *wireData)
{
        int *temp;
	u_char *pos=wireData;
	
        /* If passed a NULL pointer, return the size of the marshalled
	   thing without doing anything */
	if (wireData==NULL){
	  /* 12 bytes for handler string length, data length, active flag */
	  u_long unalign_size=(12+dataLen+strlen(handler));
	  while((unalign_size%sizeof(u_long))!=0){
	    unalign_size++;
	  }
	  return(unalign_size);
	}

	/* Write length of handler string */
	temp=(int *) pos;
	*temp=htonl(strlen(handler));
	pos+=sizeof(strlen(handler));

	/* Write handler string */
	memcpy(pos, handler, strlen(handler));
	pos+=strlen(handler);

	/* To make sure things are word-aligned, write some additional bytes */
	while((((u_long) pos-(u_long) wireData) % sizeof(u_long))!=0){
	  *pos=0;
	  pos++;
	}

	/* Write active flag */
	temp=(int *) pos;
	*temp=htonl(active);
	pos+=sizeof(active);

	/* Write length of data */
	temp=(int *) pos;
	*temp=htonl(dataLen);
	pos+=sizeof(dataLen);

	/* Write data */
	memcpy(pos, data, dataLen);
	pos+=dataLen;

	return ((u_long) pos-(u_long) wireData);
}

u_long Msg::Unmarshal(u_char *wireData)
{
	u_char *pos=wireData;
	int *temp;
	int strLen;
	
	/* Read length of handler string */
	temp=(int *) pos;
	strLen=ntohl(*temp);
	pos+=sizeof(strLen);

	/* Read handler string */
	memcpy(handler, pos, strLen);
	handler[strLen]='\0';
	pos+=strLen;

	/* To make sure things are word-aligned, read some additional bytes */
	while((((u_long) pos-(u_long) wireData) % sizeof(u_long))!=0){
	  pos++;
	}

	/* Read active flag */
	temp=(int *) pos;
	active=ntohl(*temp);
	pos+=sizeof(active);

	/* Read length of data */
	temp=(int *) pos;
	dataLen=ntohl(*temp);
	pos+=sizeof(dataLen);

	/* Read data */
	data = (unsigned char *) malloc(dataLen);
	memcpy(data, pos, dataLen);
	pos+=dataLen;

	return ((u_long) pos-(u_long) wireData);
	
}

SendableSpandObject *Msg::ToObject()
{
  SpandObjectFactory f;
  SendableSpandObject *o;
  
  /* XXX Assumes no active messages */
  o=f.MakeObject(handler);
  o->Unmarshal(data);

  return o;
    
}
  
Flow::Flow()
{
}

/* Initializes a flow that already exists */
void Flow::Init(FlowID id0)
{
	
	((ReportPayload *) this)->Init();
	id=id0;
	startSeq=0;
	startTime.tv_sec=0;
	startTime.tv_usec=0;
}

/* Prepares Flow structure */
u_long Flow::Unmarshal(u_char *wireData)
{
	u_char *p=wireData;
	struct WireFormat *m;

	/* First Unmarshal ReportPayload */
	p+=ReportPayload::Unmarshal(p);
	
	m=(struct WireFormat *) p;
	p+=sizeof(struct WireFormat);
	startSeq=ntohl(m->startSeq);
	lastSeq=ntohl(m->startSeq);
	startTime.tv_sec=ntohl(m->startTime.tv_sec);
	startTime.tv_usec=ntohl(m->startTime.tv_usec);
	
	return (u_long) p-(u_long) wireData;
}

/* Prepares Flow structure */
u_long Flow::Marshal(u_char *wireData)
{
	u_char *p=wireData;
	struct WireFormat *m;

        /* If passed a NULL pointer, return the size of the marshalled
	   thing without doing anything */
	if (wireData==NULL){
		return ReportPayload::Marshal(NULL)+sizeof(struct WireFormat);
	}
	
	/* First Marshal ReportPayload */
	p+=ReportPayload::Marshal(p);
	
	m=(struct WireFormat *) p;
	p+=sizeof(struct WireFormat);
	m->startSeq=htonl(startSeq);
	m->startSeq=htonl(lastSeq);
	m->startTime.tv_sec=htonl(startTime.tv_sec);
	m->startTime.tv_usec=htonl(startTime.tv_usec);
	
	return (u_long) p-(u_long) wireData;
}

Requestor::ConnectStreamSpand(){
  requestSock = new TcpSocket();
  requestSock->SetCompatible(1);
  requestSock->Connect(servername, REQUEST_PORT);
}

Requestor::ConnectDgramSpand(){
  requestSock = new UdpSocket();
  requestSock->SetCompatible(1);
  requestSock->Connect(servername, REQUEST_PORT);
  requestSock->SetTimeout(1000000);
}

/* This object sends active messages to the reporter */
Requestor::Requestor(char *servername0)
{
  strncpy(servername, servername0, MAX_MSG_SIZE-1);
}

Requestor::~Requestor()
{
	/* Free socket for reports */
	delete requestSock;
}  

/* Make a request */
Msg *Requestor::Request(Msg *m, int waitForReply)
{
  int size;

  if (m==NULL){
    return NULL;
  }

  size=m->Marshal(NULL);

  if (size>=MAX_MSG_SIZE){
    return RequestStream(*m, waitForReply);
  } else { 
    return RequestDgram(*m, waitForReply);
  }
}

Msg *Requestor::RequestDgram(Msg &m, int waitForReply)
{
  int size;
  Msg *replyMsg=NULL;
  u_char *buf;
  int numRetries=5;
  Collection<Socket *> socks;
  
  if (requestSock==NULL){
    ConnectDgramSpand();
  } else if (requestSock!=NULL && requestSock->GetType()!=SOCK_UDP){
    delete requestSock;
    ConnectDgramSpand();
  }

  size=m.Marshal(NULL);
  buf = (u_char *) malloc (size*sizeof(u_char));
  m.Marshal(buf);
  socks.Add(&requestSock);

  if (requestSock->Send((char *) buf, size)==SOCK_FAILURE){
    perror("Send failed\n");
    debug_perror(1, "Error while sending reply to requestor");
  }

  if (waitForReply){
    while (numRetries>0){

      /* Wait for a reply */
      if (Select("r", 1000000, socks)!=NULL){
	replyMsg = new Msg((UdpSocket *) requestSock);
	numRetries=0;
      } else {
	numRetries--;
	if (requestSock->Send((char *) buf, size)==SOCK_FAILURE){
	  perror("Send failed\n");
	  debug_perror(1, "Error while sending reply to requestor");
	}
      }
    }
    free(buf);
  
    if (replyMsg==NULL){
      return RequestStream(m, waitForReply);
    }

    return replyMsg;
  } else {
    return NULL;
  }
}

Msg *Requestor::RequestStream(Msg &m, int waitForReply)
{
  int size;
  u_char *buf;
  Msg *replyMsg=NULL;
  
  if (requestSock==NULL){
    ConnectStreamSpand();
  } else if (requestSock!=NULL && requestSock->GetType()!=SOCK_UDP){
    delete requestSock;
    ConnectStreamSpand();
  }

  size=m.Marshal(NULL);
  buf = (u_char *) malloc(size*sizeof(u_char));
  m.Marshal(buf);

  if (requestSock->Send((char *) buf, size)==SOCK_FAILURE){
    fprintf(stderr, "Send failed\n");
    debug_perror(1, "Error while making request");
  }

  if (waitForReply){
    replyMsg = new Msg(requestSock);
    free(buf);
    return replyMsg;
  } else {
    return(NULL);
  }
}

SendableSpandObject *SpandObjectFactory::MakeObject(char *name)
{
  if (strncmp(name, "spand.Addr", strlen("spand.Addr"))==0){
    return new Addr();
  } else if (strncmp(name, "spand.ReportPort", strlen("spand.ReportPort"))==0){
    return new ReportPort();
  } else if (strncmp(name, "spand.AddrPort", strlen("spand.AddrPort"))==0){
    return new AddrPort();
  } else if (strncmp(name, "spand.ReportPayload", strlen("spand.ReportPayload"))==0){
    return new ReportPayload();
  } else if (strncmp(name, "spand.PerfRequestPayload", strlen("spand.PerfRequestPayload"))==0){
    return new PerfRequestPayload();
  } else if (strncmp(name, "spand.PerfResponsePayload", strlen("spand.PerfResponsePayload"))==0){
    return new PerfResponsePayload();
  } else if (strncmp(name, "spand.SummRequestPayload", strlen("spand.SummRequestPayload"))==0){
    return new SummRequestPayload();
  } else if (strncmp(name, "spand.SummResponsePayload", strlen("spand.SummResponsePayload"))==0){
    return new SummResponsePayload();
  } else if (strncmp(name, "spand.RawRequestPayload", strlen("spand.RawRequestPayload"))==0){
    return new RawRequestPayload();
  } else if (strncmp(name, "spand.ErrorPayload", strlen("spand.ErrorPayload"))==0){
    return new ErrorPayload();
  } else if (strncmp(name, "spand.SendableSpandObjectColl", strlen("spand.SendableSpandObjectColl"))==0){
    return new SendableSpandObjectColl();
  } else if (strncmp(name, "spand.Msg", strlen("spand.Msg"))==0){
    return new Msg();
  } else if (strncmp(name, "spand.Flow", strlen("spand.Flow"))==0){
    return new Flow();
  } else {
    fprintf(stderr, "Don't know about class %s\n");
    return(NULL);
  }

}


SendableSpandObjectColl::SendableSpandObjectColl()
{
  sprintf(name, "");
  printBuf = (char *) malloc(MAX_MSG_SIZE*sizeof(char));
}

SendableSpandObjectColl::~SendableSpandObjectColl()
{
  SendableSpandObject *o;
  SendableSpandObject **op;
  
  for (op=objects.First(); op!=NULL; op=objects.Next()){
    o=*op;
    delete(o);
  }
  free(printBuf);
}


u_long SendableSpandObjectColl::Marshal(u_char *wireData)
{
  int i;
  int *temp;
  u_char *pos=wireData;
  SendableSpandObject *o;
  SendableSpandObject **op;      

  if (wireData==NULL){
    int size=sizeof(objects.nElem)+sizeof(strlen(name))+strlen(name);
    while((size%sizeof(u_long))!=0){
      size++;
    }
    for (op=objects.First(); op!=NULL; op=objects.Next()){
      o=*op;
      size+=o->Marshal(NULL);
    }
    return(size);
  }

  /* Write length of type string */
  temp=(int *) pos;
  *temp=htonl(strlen(name));
  pos+=sizeof(strlen(name));
  
  /* Write the type string */
  memcpy(pos, name, strlen(name));
  pos+=strlen(name);

  /* To make sure things are word-aligned, write some additional bytes */
  while((((u_long) pos-(u_long) wireData) % sizeof(u_long))!=0){
    *pos=0;
    pos++;
  }

  /* Write the number of Objects */
  temp=(int *) pos;
  *temp=htonl(objects.nElem);
  pos+=4;

  /* Loop over objects, writing each */
  for(op=objects.First(); op!=NULL; op=objects.Next()){
    o=*op;
    pos+=o->Marshal(pos);
  }

  return (pos-wireData);
}

u_long SendableSpandObjectColl::Unmarshal(u_char *wireData)
{

  int i;
  int *temp;
  u_char *pos=wireData;
  int strLen;
  int count;
  SpandObjectFactory f;
  
  /* Read length of type string */
  temp=(int *) pos;
  strLen=htonl(*temp);
  pos+=sizeof(strLen);
  
  /* Read the name string */
  memcpy(name, pos, strLen);
  pos+=strLen;
  name[strLen]='\0';

  /* To make sure things are word-aligned, read some additional bytes */
  while((((u_long) pos-(u_long) wireData) % sizeof(u_long))!=0){
    pos++;
  }

  /* Read the number of Objects */
  temp=(int *) pos;
  count=htonl(*temp);
  pos+=sizeof(count);

  /* Loop over objects, reading each */
  for(i=0; i<count; i++){
    SendableSpandObject *o =f.MakeObject(name);
    pos+=o->Unmarshal(pos);
    objects.Add(&o);
  }

  return (pos-wireData);

}

char *SendableSpandObjectColl::Print()
{

  SendableSpandObject *o;
  SendableSpandObject **op;
  char tmpBuf[MAX_MSG_SIZE];
  
  int size=objects.nElem*MAX_MSG_SIZE-1;

  printBuf=(char *) realloc(printBuf, (objects.nElem+1)*MAX_MSG_SIZE*sizeof(char));
  strncpy(printBuf, "", size);
  
  /* Print name */
  sprintf(tmpBuf, "%s %d\n", name, objects.nElem);
  size-=strlen(tmpBuf);
  strncat(printBuf, tmpBuf, size);

  /* Print number of objects */
  for (op=objects.First(); op!=NULL && size>0; op=objects.Next()){
    o=*op;
    strncat(printBuf, o->Print(), size);
    size-=strlen(o->Print());
    strncat(printBuf, " \n", size);
    size--;
  }
  return printBuf;

}
