/*
 * Copyright (c) 1996-1998 Regents of the University of California.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the SPAND Research
 *      Group at the University of California Berkeley.
 * 4. Neither the name of the University nor of the Research Group may be
 *    used to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
/* $Header: /home/stemm/cvs/porting/porting.h,v 1.4 1998/06/10 23:24:13 stemm Exp $ */
#ifndef OSFLAGS
#include <sys/types.h>
#include <netinet/in.h>

/** @name porting library
    This library is used to ease porting across UNIXen.
 * @author SPAND Group, UC Berkeley.
 * @version $Revision: 1.4 $.
 */
//@{

/** Minimum parameter to usleep.  */
#define MIN_USLEEP 1500

/** Define either RANDOM or RAND to use random() instead of rand().   */
#define RANDOM

/** Define one of USLEEP or NANOSLEEP if your system has it, program uses 
   sleep by default.   */
#define USLEEP

/** Define MULTICAST if support is available.   */
#ifdef IP_ADD_MEMBERSHIP
#define MULTICAST
#endif

/** Define TOS if TOS sockopts are available.  */
#define TOS

/** Define THREADED if you want to use the MT safe calls.  */

/*useful defaults for various systems */

#ifdef OSF1
#endif

#ifdef AIX4
#include <sys/select.h>
#include <unistd.h>
#include <sys/uio.h>
#endif

#ifdef AIX3
#include <sys/select.h>
#include <unistd.h>
#include <sys/uio.h>
#endif

#ifdef SunOS
#undef TOS
#endif

#ifdef Solaris
#undef TOS
#undef RANDOM
#undef USLEEP
#define RAND
#define NANOSLEEP
#define THREADED
#endif

#ifdef HPUX
#undef TOS
#undef RANDOM
#define RAND
#undef USLEEP
#endif

#ifdef ULTRIX
#undef TOS
#endif

#ifdef BSDI
#include <sys/unistd.h>
#include <sys/uio.h>
#ifdef TCPSTATS
#define TCPSTATS
#define TCP_ENABLE_STATS  0x800
#define TCP_MONITOR
#define TCP_GET_SRTT            0x800000 /* get srtt from kernel */
#define TCP_GET_RTTVAR          0x1000000/* get rtt variance from kernel */
#include "/sys/netinet/tcp_stats.h"
#endif
#endif

#if defined(__cplusplus)
extern "C" {
  /** Sleep for the specified number of microseconds.  */
    void microsleep(int usec);
}
#else
extern void microsleep(int usec);
#endif

//@}
#endif
