/*
 * Copyright (c) 1996-1998 Regents of the University of California.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the SPAND Research
 *      Group at the University of California Berkeley.
 * 4. Neither the name of the University nor of the Research Group may be
 *    used to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
/* $Header: /home/stemm/cvs/SPAND/src/snooper/handlers.cc,v 1.22 1998/06/26 21:19:31 stemm Exp $ */
#include <stdio.h>
#include <string.h>
#include <sys/time.h>
#include <math.h>
#include <strings.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <unistd.h>

#include <netinet/in.h>
#include <netinet/in_systm.h>
#if defined(BSDI) || defined(Solaris)
#include <sys/socket.h>
#include <net/if.h>
#include <net/if_arp.h>
#endif
#include <netinet/if_ether.h>
#include <netinet/ip.h>
#include <netinet/tcp.h>
#if defined(BSDI) || defined(Solaris)
#include <netinet/ip_var.h>
#include <netinet/tcpip.h>
#endif
#include <pcap.h>
#include <Spand.h>
#ifdef AIX
#include <netinet/if_802_5.h>
#endif
#include "handlers.h"

/* If 1, then generate perf requests at start of each connection and
   compare actual perf to that returned by aggregator */

extern int test_aggregator;
extern char *spandSvr;
extern int warmThresh;
extern int minLen;

const u_char *snapend;

/* Format of socks packet is: */
/*
  +----+----+----+----+----+----+----+----+----+----+....+----+
  | VN | CD | DSTPORT |      DSTIP        | USERID       |NULL|
  +----+----+----+----+----+----+----+----+----+----+....+----+
  # of bytes:    1    1      2              4           variable        1
  
  For Socks 4, VN should be 4.
  For Connect request, CD should be 1.
  */
int socks_handler(const u_char *packet, u_long length, FlowID &id)
{
    /* If length is less than 8, then can't find encapsulated host */
    if (length<8){
	return -1;
    }
  
    /* VN should be 4 and CD should be 1 */
    if (packet[0]!=4 || packet[1]!=1){
	return -1;
    }
  
    id.dst.port.Unmarshal((u_char *) (packet+2));
    id.dst.addr.Unmarshal((u_char *) (packet+4));
  
    return 0;
}

int ipip_handler(const u_char *packet, u_long length, FlowID &id)
{
}

int http11proxy_handler(const u_char *packet, u_long length, FlowID &id)
{
}

typedef enum {ENCAP_IP=0, ENCAP_SOCKS, ENCAP_HTTP11PROXY} Encapsulation_Type;

/* An encapsulation handler takes as argument the packet contents and
   modifies the flow ID argument to match the actual FlowID. If
   returns 0, then the flow ID has been modified. If 1, then the FlowID
   has not been modified. */

typedef int (*EncapHandler)(const u_char *packet, u_long size, FlowID &id);

/* A configuration file maps from destination port and address to
   encapsulation type, so we can identify proxy hosts */
class Proxy {
public:
    AddrPort a;
    Encapsulation_Type type;
};

class ProxyList {
public:
    ProxyList(){ numProxies=0;};
    Proxy p[1000];
    int numProxies;
};

/* List of encapsulation handlers. Order of handlers is same as order of Encapsulation_Type enums */
static EncapHandler encapHandlers[] = {
    ipip_handler, /* ENCAP_IPIP */
    socks_handler, /* ENCAP_SOCKS */
    http11proxy_handler /* ENCAP_HTTP11PROXY */
};

/* Garbage collects, removing flows that have lasted longer than max_age seconds */
void garbage_collect(Hash<FlowID, Flow> &hash,
		     u_long max_age,
		     const struct timeval &now,
		     int *flows,
		     u_char *flags,
		     Requestor &requestor)
{
  Flow *f;
  u_long deletedFlows=0;

  fprintf(stderr, "Garbage Collecting...");
  // Sequentially loop through hash table
  for(f=hash.First(); f!=NULL; f=hash.Next()){
    if (now.tv_sec-f->lastPktTime.tv_sec>max_age){
      /* We garbage collect by making a report using the last received
         packet as the end of the connection */
      /*      fprintf(stdout, "r %s start=%u end=%u length=%u dur=%u\n",
	      f->Print(), f->startSeq, f->lastSeq,
	      f->sampLen, f->sampDur);

      requestor.Report(*f);*/
      hash.Del(f->id);
      deletedFlows++;
      //      fprintf(stderr, "Deleting flow %s\n", f->id.Print());
      delete f;
    }
    fprintf(stderr, "Deleted %d flows\n", deletedFlows);
  }
}

/* Parses ip header. If return code !=0, then not a valid IP packet */
int parse_tcpip_header(const struct pcap_pkthdr *h,
		       const u_char **packet,
		       u_int *length,
		       FlowID &id,
		       u_char *flags,
		       u_long *seq,
		       u_long *ack,
		       u_long *payload_len)
{
    register const struct ip *ip;
    register const struct tcphdr *tp;
    ip = (const struct ip *) (*packet);
    u_long hlen, len, off;
  
    if (ip->ip_v!=0x4){
	return -1;
    }
  
    if ((u_char *)(ip + 1) > snapend ||
	(*length) < sizeof (struct ip)) {
	return -1;
    }
  
    hlen = ip->ip_hl * 4;
  
    len = ntohs(ip->ip_len);
    len -= hlen;
  
    off = ntohs(ip->ip_off);
    if ((off & 0x1fff) != 0) {
	return -1;
    }
  
    (*packet) = (*packet) + hlen;
    (*length) = (*length) - hlen;
    if (ip->ip_p!=IPPROTO_TCP) {
	return -1;
    }
  
    /* If truncated tcp packet, ignore */
    if ((*length) < sizeof(*tp)) {
	return -1;
    }
  
    tp=(struct tcphdr *) (*packet);

    id.src.addr.addr.s_addr=ntohl(ip->ip_src.s_addr);
    id.dst.addr.addr.s_addr=ntohl(ip->ip_dst.s_addr);
    id.src.port.port=ntohs(tp->th_sport);
    id.dst.port.port=ntohs(tp->th_dport);
  
    (*flags)=tp->th_flags;
    (*ack)=ntohl(tp->th_ack);
    (*seq)=ntohl(tp->th_seq);
  
    /* Skip past tcp header to start of packet */
    (*packet) = (*packet)+(tp->th_off*4);
    (*length) = (*length)-(tp->th_off*4);
    (*payload_len) = len -tp->th_off*4;
    return 0;
}

int ReadHostFile(char *hostfilename, ProxyList &proxyList)
{
    FILE *hostFile;
    char addr[100];
  
    if ((hostFile=fopen(hostfilename, "r"))==NULL){
	perror("Couldn't open host file\n");
	return(-1);
    }
  
    while (!feof(hostFile)){
	Proxy *p=&(proxyList.p[(proxyList.numProxies)]);
	fscanf(hostFile, "%s %d %d\n", addr, &(p->a.port), &(p->type));
	p->a.addr.addr.s_addr=ntohl(inet_addr(addr));
	proxyList.numProxies++;
    }
  
    fclose(hostFile);
}

void make_report(int flags,
		 Flow *flow,
		 int *numReports,
		 int *flows,
		 int *oldFlows,
		 u_long seq,
		 u_long payload_len,
		 Requestor &requestor,
		 Hash<FlowID, Flow> &hash)
{
  u_long bits=(flow->sampLen)*8;
  float sec=(float) flow->sampDur/(float) 1000.0;
  float bw = (float) bits/sec;
  Msg *perfResponse;
  
  if (!(flags & TH_RST)){
    if (seq+payload_len>flow->lastSeq){
      flow->sampLen=seq+payload_len-flow->startSeq;
    } else {
	flow->sampLen=flow->lastSeq-flow->startSeq;
    }
  }
  
  if (flow->sampLen>minLen && flow->sampDur>0){
    if (strcmp(flow->id.dst.Print(), "198.81.209.16.3823")==0){
      bw++;
      bw--;
    }
    
    fprintf(stdout, "r %s start=%u end=%u length=%u dur=%u bw=%f\n",
	    flow->Print(), flow->startSeq, flow->lastSeq,
	    flow->sampLen, flow->sampDur, bw);
    // Only Make a request if have received warmThresh reports and if we should
    if (test_aggregator && (*numReports)>warmThresh) {
      PerfRequestPayload payload(flow->id.src.addr);
      Requestor requestor("127.0.0.1");
      perfResponse=requestor.Request(new Msg("spand.GetPerf", payload, 1), 1);
      float bits = flow->sampLen*8;
      float sec = (float) flow->sampDur/(float) 1000;
      float bw = (u_long) ((float) bits)/((float) sec);
      if (perfResponse!=NULL){
	PerfResponsePayload *p = new PerfResponsePayload();
	p->Unmarshal(perfResponse->data);
	if (p != NULL) {
	  float lowCI = p->availBandwidth-1.65*p->stdDevAvailBandwidth;
	  float highCI = p->availBandwidth+1.65*p->stdDevAvailBandwidth;
	  
	  float ratio=(float) p->availBandwidth/bw;
	  fprintf(stderr, "%s predicted %u 90%% CI %f %f actual %u inCI %d ratio %f numReports %u Avgsrtt %u actual %u\n",
		  flow->id.Print(),
		  p->availBandwidth,
		  lowCI, highCI,
		  (u_long) bw,
		  (bw>=lowCI && bw<=highCI)? 1:0,
		  ratio, p->numReports, p->avgSrtt, flow->srtt);
	  fflush(stderr);
	  delete p;
	} else {
	  fprintf(stderr, "predicted none actual %u\n", bw);
	}
      }
      delete perfResponse;
    }

    /* Add this report to the database */
    requestor.Request(new Msg("spand.AddReport", *flow, true), 0);

    hash.Del(flow->id);
  
    delete flow;
    (*flows)=(*flows)-1;
    if ((*flows)-(*oldFlows)>10){
      fprintf(stderr, "%d flows (%d bytes)\n", (*flows), (*flows)*sizeof(Flow));
      (*oldFlows)=(*flows);
    }
    (*numReports)=(*numReports)+1;
  }
}

Flow *init_flow(FlowID &id,
		Hash<FlowID, Flow> &hash,
		u_long seq,
		u_long payload_len,
		const struct pcap_pkthdr *h,
		int *flows,
		int *oldFlows)
{
    Flow *flow;
  
  flow = new Flow(id);

  flow->startSeq=seq;
  flow->lastSeq=seq+payload_len;
  flow->maxSeq=seq;
  flow->rttTs.tv_sec=h->ts.tv_sec;
  flow->rttTs.tv_usec=h->ts.tv_usec;
  flow->rttSeq=seq;
  flow->startTime.tv_sec=h->ts.tv_sec;
  flow->startTime.tv_usec=h->ts.tv_usec;
  flow->lastPktTime.tv_sec=h->ts.tv_sec;
  flow->lastPktTime.tv_usec=h->ts.tv_usec;
  
  /* Put this flow in the database and delete our local copy */
  hash.Put(flow->id, flow);
    
  flow = hash.Get(flow->id);
  if (flow==NULL){
    fprintf(stderr, "Didn't find the flow we just put in?\n");
  }
    
  (*flows)++;
  if ((*flows)-(*oldFlows)>10){
    fprintf(stderr, "%d flows (%d bytes)\n", (*flows), (*flows)*sizeof(Flow));
    (*oldFlows)=(*flows);
  }
  return flow;
}

void tcpip_handler(const struct pcap_pkthdr *h, const u_char *packet, u_int length)
{
  
  static int didInit=0;
  static Requestor requestor(spandSvr);

  Flow *flow;
  int rc;
  struct timeval diffTime, gapTime;
  FlowID id;
  u_char flags;
  u_long seq, ack, payload_len;
  static ProxyList proxyList;
  int i;
  static Hash<FlowID, Flow> hash("/tmp/flows.db");
  static int flows=0, oldFlows=0;
  AddrPort temp;
  FlowID tempID;
  Flow *oppFlow;
  FlowID oppTempID;
  int found;
  static u_long gcInterval=5*60; // Garbage collection frequency
  static timeval lastGCTime;      // Last Time Garbage collection was done
  static int numReports=0;
  
  /* Open database if not yet opened */
  if (!didInit){
    didInit=1;
    
    /* Also read the file containing (encapsulation host,port,type) triples */
    ReadHostFile("encaphosts.conf", proxyList);

    /* Initialize the last Garbage Collection Time */
    lastGCTime.tv_sec=0;
    lastGCTime.tv_usec=0;

  }

  /* Parse TCP/IP header to get source and dest addr/port, flags, and ack fields */
  if (parse_tcpip_header(h, &packet, &length, id, &flags, &seq, &ack, &payload_len)){
    return;
  }

  /* If more than gcInterval seconds have passed since the last time
     we garbage collected, then Garbage Collect */
  if (h->ts.tv_sec-lastGCTime.tv_sec>gcInterval){
    garbage_collect(hash, gcInterval, h->ts, &flows, &flags, requestor);
    lastGCTime.tv_sec=h->ts.tv_sec;
    lastGCTime.tv_usec=h->ts.tv_usec;
  }

  /* If here, then we have a valid TCP/IP packet. packet points to
       start of data part of packet, and length is length of the data
       part. */
    if (flags & TH_SYN){
	fprintf(stdout, "s %s\n", id.Print());
    }
  
    if ((flags & TH_FIN)){
	fprintf(stdout, "f %s\n", id.Print());
    }

    /* If we see a RST, we treat that as a FIN from the opposite
       direction of the connection */
    if ((flags & TH_RST)) {
    
	/* Set the FIN flag */
	flags |= TH_FIN;
    
	/* Set the sequence number to be the last sequence number seen
           in the opposite direction */
    
	fprintf(stdout, "R %s\n", id.Print());
	temp=id.src;
	id.src=id.dst;
	id.dst=temp;
    }
  
    /* Find the flow in the HashTable */
    /* We assume that a flow starts when a syn is seen from the source
       and ends when a fin is seen from the source. If we see a syn and the
       flow already exists in the database, then something is wrong (we
       shouldn't see this) and we ignore this packet. Similarly, if we see
       a non-syn (data, fin, or reset) packet and the session does not
       exist in the database, we assume that we missed the beginning of the
       connection and ignore the packet. */
    flow=hash.Get(id);
    
    if (flow!=NULL && (flags & TH_SYN)){
	fprintf(stdout, "Multiple syn? %s\n", id.Print());
	return;
    }
  
    if (flow==NULL){
	if (!(flags & TH_SYN)){
	    return;
	}
    
	/* If SYN was set, beginning of flow, must create new flow object */
    
	flow=init_flow(id, hash, seq, payload_len, h, &flows, &oldFlows);
    }
  
    /* If here, then object was found in database */
    tempID=flow->id;
  
    oppTempID.src=flow->id.dst;
    oppTempID.dst=flow->id.src;
    oppFlow=hash.Get(oppTempID);

    /* If the dest addr and port is one in the encap list, then call
       the specified encap handler */
    for(i=0; i<proxyList.numProxies; i++){
	Proxy *p=&(proxyList.p[i]);
	if (flow->id.dst.addr==p->a.addr &&
	    flow->id.dst.port==p->a.port &&
	    (rc=encapHandlers[(int) p->type](packet, length, tempID))==0){
      
	    /* The encap handler found a new source or destination. We
	       must also change the opposite direction to match the new
	       source and dest. */
      
	    oppTempID.src=flow->id.dst;
	    oppTempID.dst=flow->id.src;
      
	    flow->id=tempID;
      
      /* Also find the opposite direction, if one exists */
      oppFlow=hash.Get(oppTempID);
      if (oppFlow!=NULL){
	oppFlow->id.src=flow->id.dst;
	oppFlow->id.dst=flow->id.src;
      }
    }
  }

  gapTime.tv_usec=h->ts.tv_usec-flow->lastPktTime.tv_usec;
  gapTime.tv_sec=h->ts.tv_sec-flow->lastPktTime.tv_sec;
  if (gapTime.tv_usec<0){
    gapTime.tv_usec+=1000000;
    gapTime.tv_sec-=1;
  }

  /* If more than thinkTime (10) seconds have passed since the last
     packet on this flow, then make a report for this flow and
     re-initialize the variables for the flow */
  if (gapTime.tv_sec>=5){
    make_report(flags, flow, &numReports, &flows, &oldFlows, seq, payload_len, requestor, hash);
    flow=init_flow(id, hash, seq, payload_len, h, &flows, &oldFlows);
    
    } else {

	flow->lastPktTime.tv_sec=h->ts.tv_sec;
	flow->lastPktTime.tv_usec=h->ts.tv_usec;
	if (seq > flow->maxSeq) {
	    flow->maxSeq = seq;
	    if (flow->rttTs.tv_sec == 0) {
		flow->rttSeq = seq;
		flow->rttTs.tv_sec = h->ts.tv_sec;
		flow->rttTs.tv_usec = h->ts.tv_usec;
	    }
	} else {
	    // reset rtt timer due to rexmit
	    if (flow->maxSeq == seq) {
		flow->rttTs.tv_sec = h->ts.tv_sec;
		flow->rttTs.tv_usec = h->ts.tv_usec;
	    } else
		flow->rttTs.tv_sec = 0;
	}
	/* Update variables in session */
	if (!(flags & TH_RST)){
	    if (flow->lastSeq<seq+payload_len){
		flow->lastSeq=seq+payload_len;
	    }
	}
	flow->sampLen=seq-flow->startSeq;

    /* Update variables in session */
    if (!(flags & TH_RST)){
      if (flow->lastSeq<seq+payload_len){
	flow->lastSeq=seq+payload_len;
      }
    }
    flow->sampLen=seq-flow->startSeq;

    diffTime.tv_usec=h->ts.tv_usec-flow->startTime.tv_usec;
    diffTime.tv_sec=h->ts.tv_sec-flow->startTime.tv_sec;
    if (diffTime.tv_usec<0){
      diffTime.tv_usec+=1000000;
      diffTime.tv_sec-=1;
    }
  
    flow->sampDur=diffTime.tv_sec*1000+diffTime.tv_usec/1000;
    flow->packetsReceived++;
  
    flow->mostSigNTP=h->ts.tv_sec;
    flow->leastSigNTP=h->ts.tv_usec;
  
    /* XXX Packet size, packetsLost always 0 for now */
    flow->packetSize=0;
    flow->packetsLost=0;
    if (oppFlow != NULL && ack > oppFlow->rttSeq) {
 	    if (oppFlow->rttTs.tv_sec != 0) {
 		int cur_rtt = (h->ts.tv_sec - oppFlow->rttTs.tv_sec) * 1000000+
 		    (h->ts.tv_usec - oppFlow->rttTs.tv_usec);
		if (oppFlow->srtt == 0) 
 		    oppFlow->srtt = cur_rtt;
 		else
 		    oppFlow->srtt = (.875 * oppFlow->srtt) + (.125 * cur_rtt);
	    }
 	    oppFlow->rttTs.tv_sec = 0;
 	    oppFlow->rttTs.tv_usec = oppFlow->lastPktTime.tv_usec;
 	    oppFlow->rttSeq = oppFlow->lastSeq;
 	}
  }
  
  /* If FIN/RST was set, session is over, print statistics and remove flow from hash */
  // For now, only report if fin was seen
  //  if ((flags & TH_FIN) || (flags & TH_RST)){
  if (flags & TH_FIN){
    char *foo = flow->id.src.Print();
    if (strstr(&foo[strlen(foo) - 3], ".80") != NULL && oppFlow != NULL) {
      flow->srtt = oppFlow->srtt;  // a fix for server srtt
    }
    make_report(flags, flow, &numReports, &flows, &oldFlows, seq, payload_len, requestor, hash);
  } else {
    /* Put the flow back into the database */
    /*		hash.Put(flow->id, flow);*/
  }
  
  return;
}

void ether_if_handler(u_char *user, const struct pcap_pkthdr *h, const u_char *p)
{
    u_int caplen = h->caplen;
    u_int length = h->len;
  
    snapend = p + caplen;
  
    if (caplen >= sizeof(struct ether_header)){
	p += sizeof(ether_header);
	caplen -= sizeof(ether_header);
	tcpip_handler(h, p, caplen);
    }
}

#ifdef AIX4
#include <netinet/if_802_5.h>
void token_if_handler(u_char *user, const struct pcap_pkthdr *h, const u_char *p)
{
    u_int caplen = h->caplen;
    u_int length = h->len;
    u_int link_hdr_size;
  
    snapend = p + caplen;
    link_hdr_size=mac_size((ie5_mac_hdr *) p) + sizeof(struct ie2_llc_snaphdr);
    /* link_hdr_size=14; */
  
    if (caplen >= link_hdr_size){
	p += link_hdr_size;
	caplen -= link_hdr_size;
	tcpip_handler(h, p, caplen);
    }
}
#endif

